import type { InferSchemaType } from 'mongoose';
import mongoose, { Schema } from 'mongoose';
import { ReconciliationTypes } from './types';

const schema = new Schema(
    {
        organisationId: { type: Number, required: true },
        queryExecutionId: { type: String, required: true },
        fromDate: { type: Date, required: true },
        toDate: { type: Date, required: true },
        metricType: {
            type: String,
            enum: [
                ReconciliationTypes.Interaction,
                ReconciliationTypes.V2Interaction,
                ReconciliationTypes.Workstream,
                ReconciliationTypes.TaskData,
            ],
            required: true,
        },
    },
    {
        collection: 'athena_query_metadata',
    },
);

schema.index({
    organisationId: 1,
    requestId: 1,
});

export type AthenaQueryMetadata = InferSchemaType<typeof schema>;

export const AthenaQueryMetadataModel = mongoose.model('AthenaQueryMetadataModel', schema);
