type MongoLocation = 'AU-NSW' | 'AU-VIC' | 'IE' | 'US-OR';

const getMongoLocationFromAWS = (region: string | undefined): MongoLocation => {
    switch (region) {
        case 'ap-southeast-2':
            return 'AU-NSW';
        case 'us-west-2':
            return 'US-OR';
        case 'eu-west-1':
            return 'IE';
        default:
            return 'AU-VIC';
    }
};

export const config = {
    log: {
        log_level: process.env['LOG_LEVEL'] ?? 'info',
    },
    agyletime: {
        api_gateway_base_url: process.env['API_GATEWAY_URL'] ?? 'http://localhost',
        clusterName: process.env['CLUSTER_NAME'],
    },
    performance: {
        syncPerformanceViewsQueueUrl: process.env['SYNC_PERF_VIEWS_QUEUE_URL'] ?? '',
        syncTaskDefinitionQueueUrl: process.env['SYNC_TASK_DEFINITION_QUEUE_URL'],
    },
    db: {
        url: process.env['MONGO_DB_URL'] ?? 'mongodb://localhost:27017',
        name: 'performance',
        location: getMongoLocationFromAWS(process.env['AWS_REGION']),
    },
    memoryServer: {
        url: 'mongodb://localhost:27018',
        name: 'performance',
    },
    aws: {
        region: process.env['AWS_REGION'] ?? '',
    },
    featureflow: {
        apiKey: process.env['FEATUREFLOW_API_KEY'] ?? '',
    },
    commonHandlerParams: {
        squadName: 'cristork',
        repoName: 'performance-commands',
    },
    mongoBucketSizeInMinutes: 15,
    athena: {
        db: process.env['ATHENA_DB'],
        athenaQueriesBucket: process.env['ATHENA_QUERY_BUCKET'],
    },
    featureFlags: {
        orgAllowListTPM: process.env['PVF_TPM_ALLOW_LIST'] ?? 'none',
    },
    stepFunctions: {
        delayInMinutes: parseInt(process.env['SF_BATCH_GROUP_DELAY_MINUTES'] ?? '2'),
        batchGroupSize: parseInt(process.env['SF_BATCH_GROUP_SIZE'] ?? '100'),
    },
};
