import { BadRequestError } from '../../errors';
import { log } from '../../utils';
import { doRemoveCustomLabel } from '../doRemoveCustomLabel';
import { extractParameters } from '../extractParameters';
import { APIGatewayProxyEvent } from 'aws-lambda';
import { extractIdentity, formatAsJSend, JSend } from '@agyletime/wfm-aws-utils';

export const deleteCustomLabel = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    try {
        log.debug('DELETE %s', event.path, event);
        const identity = extractIdentity(event);
        const organisationId = Number(identity.organisationId);
        const { field, category, subCategory } = extractParameters(event);

        if (!field) {
            throw new BadRequestError('Invalid field');
        }
        if (!category) {
            throw new BadRequestError('Invalid category');
        }
        if (!subCategory) {
            throw new BadRequestError('Invalid subCategory');
        }

        await doRemoveCustomLabel({ organisationId, field, category, subCategory });

        return formatAsJSend({}, 204);
    } catch (error: unknown) {
        log.warn('Failed Running DELETE %s', error);
        throw error;
    }
};
