import { APIGatewayProxyEvent } from 'aws-lambda';
import { BadRequestError } from '../../errors';
import { log } from '../../utils';
import { doUpsertCustomLabel } from '../doUpsertCustomLabel';
import { extractParameters } from '../extractParameters';
import { extractIdentity, formatAsJSend, JSend } from '@agyletime/wfm-aws-utils';

export const patchCustomLabel = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    try {
        log.debug('PATCH %s', event.path, event);
        const identity = extractIdentity(event);
        const organisationId = Number(identity.organisationId);
        const { field, category, subCategory } = extractParameters(event);

        if (!field) {
            throw new BadRequestError('Invalid field');
        }
        if (!category) {
            throw new BadRequestError('Invalid category');
        }
        if (!subCategory) {
            throw new BadRequestError('Invalid subCategory');
        }

        const customLabel = JSON.parse(event.body ? event.body : '{}') as unknown;

        let label: string | undefined;
        if (typeof customLabel === 'object' && customLabel) {
            if ('label' in customLabel && typeof customLabel.label === 'string') {
                label = String(customLabel.label).trim();
            }
        }

        if (!label) {
            throw new BadRequestError('Invalid label');
        }

        await doUpsertCustomLabel({ organisationId, field, category, subCategory, label });

        return formatAsJSend({}, 202);
    } catch (error: unknown) {
        log.warn('Failed Running PATCH %s', error);
        throw error;
    }
};
