import type { SyncPerformanceViewsDto } from './doSyncPerformanceViews';
import { log } from '../../../utils';
import { formatEventAsCloudEvent, writeToQueue } from '@agyletime/wfm-aws-utils';

interface AddSyncPerformanceViewsJobsToQueueParams {
    queueUrl: string;
    organisationId: number;
    fromDate: string;
    toDate: string;
}

export const doAddSyncPerformanceViewsJobsToQueue = async ({
    queueUrl,
    organisationId,
    fromDate,
    toDate,
}: AddSyncPerformanceViewsJobsToQueueParams): Promise<string> => {
    const dto: SyncPerformanceViewsDto['eventData'] = {
        fromDate,
        toDate,
    };

    log.log('info', `Sending SQS Msg: orgId=${organisationId}`, { ...dto, queueUrl: queueUrl });

    const event = formatEventAsCloudEvent({
        organisationId,
        eventType: 'agyletime.performance.jobs.SyncPerformanceViews',
        eventSourceUri: '/performance/SyncPerformanceViews',
        data: dto,
    });

    // sqs will throw on error, catch the error upstream
    log.debug(`Sending to sqs`, { payLoad: event });
    await writeToQueue({ queueUrl, cloudEvent: event });

    return 'Message sent to queue';
};
