import { doSyncPerformanceViews } from '../doSyncPerformanceViews';
import { closeConnection } from '@agyletime/common-utils/mongo';
import { multiConnectionPool } from '@agyletime/wfm-rds-utils';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { typedJSONParse } from '@agyletime/wfm-aws-utils';
import { SQSEvent } from 'aws-lambda';
import { log } from '../../../../utils';

export const singleSyncPerformanceViews = async (event: SQSEvent): Promise<void> => {
    try {
        await multiConnectionPool.init();
    } catch (e: unknown) {
        //If we can't connect to mongo then clean up
        await multiConnectionPool.closeAll();
        await closeConnection();

        log.error(`Error connecting to Mongo: ${(e as Error).message}`);

        return;
    }

    for (const record of event.Records) {
        try {
            const body = typedJSONParse<AgyleTimeCloudEvent<{ fromDate: string; toDate: string }>>(record.body);
            const { agyletimeorganisationId: organisationId, data: eventData } = body;

            await doSyncPerformanceViews({
                eventData,
                organisationId: typeof organisationId === 'string' ? Number.parseInt(organisationId) : organisationId,
            });
        } catch (e: unknown) {
            log.error('Unexpect error in doSyncPerformanceViews', { e });
        }
    }

    await multiConnectionPool.closeAll();
    await closeConnection();
};
