import type { InteractionEventParams } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { parseISO, startOfDay } from 'date-fns';
import type { InteractionEventType } from './interactionEvent.type';
import type { UpdateOneModel } from 'mongoose';
import type { V2MongoInteractionMetric } from '@agyletime/performance-utils';

export default ({
    interaction,
    params,
}: {
    interaction: InteractionEventType;
    params: InteractionEventParams;
}): UpdateOneModel<V2MongoInteractionMetric> => {
    const interactionStartTimeAsLocal = toZonedTime(interaction.startTime, params.timezone);
    const timeBucketAsLocal = startOfDay(interactionStartTimeAsLocal);
    const timeBucketAsUtc = startOfDay(parseISO(interaction.startTime));

    return {
        filter: {
            timeBucketAsLocal,
            timeBucketAsUtc,
            organisationId: params.organisationId,
            userId: params.userId,
            teamId: params.teamId,
        },
        upsert: true,
        update: {
            $set: {
                timeBucketAsLocal,
                timeBucketAsUtc,
                organisationId: params.organisationId,
                userId: params.userId,
                teamId: params.teamId,
            },
            $push: {
                interactions: {
                    identifier: interaction.identifier,
                    workstreamDefinitionId: params.workstreamDefinitionId,
                    workstreamDefinitionName:
                        params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`,
                    taskTypeId: interaction.workloadTaskId,
                    startTime: interaction.startTime,
                    workstreamIdentifier: interaction.workstreamIdentifier,
                    isAssign: interaction.isAssign,
                    isPrivateComment: interaction.isPrivateComment,
                    isPublicComment: interaction.isPublicComment,
                    isReassign: interaction.isReassign,
                    statusChangeTo: interaction.statusChangeTo,
                    type: interaction.type,
                    workEffortSecs: interaction.workEffortSecs,
                    completionStatus: interaction.completionStatus,
                    talkTimeSecs: interaction.talkTimeSecs,
                    consultationTimeSecs: interaction.consultationTimeSecs,
                    wrapTimeSecs: interaction.wrapTimeSecs,
                    holdTimeSecs: interaction.holdTimeSecs,
                    waitTimeSecs: interaction.waitTimeSecs,
                },
            },
        },
    };
};
