import type { InteractionEventParams } from '../types';
import { toZonedTime } from 'date-fns-tz';
import type { MongoInteractionMetric } from '@agyletime/performance-utils';
import { parseISO, startOfDay } from 'date-fns';
import type { InteractionEventType } from './interactionEvent.type';
import type { UpdateOneModel } from 'mongoose';

export default ({
    interaction,
    params,
}: {
    interaction: InteractionEventType;
    params: InteractionEventParams;
}): UpdateOneModel<MongoInteractionMetric> => {
    const interactionStartTimeAsLocal = toZonedTime(interaction.startTime, params.timezone);
    const timeBucketAsLocal = startOfDay(interactionStartTimeAsLocal);
    const timeBucketAsUtc = startOfDay(parseISO(interaction.startTime));

    return {
        filter: {
            organisationId: params.organisationId,
            timeBucketAsLocal,
            timeBucketAsUtc,
            userId: params.userId,
            teamId: params.teamId,
        },
        update: {
            $pull: {
                interactions: { identifier: interaction.identifier },
            },
        },
    };
};
