import type { InteractionEventParams } from '../types';
import { toZonedTime } from 'date-fns-tz';
import { parseISO, startOfDay } from 'date-fns';
import type { InteractionEventType } from './interactionEvent.type';
import type { V2MongoInteractionMetric } from '@agyletime/performance-utils';
import type { UpdateOneModel } from 'mongoose';

export default ({
    interaction,
    params,
}: {
    interaction: InteractionEventType;
    params: InteractionEventParams;
}): UpdateOneModel<V2MongoInteractionMetric> => {
    const interactionStartTimeAsLocal = toZonedTime(interaction.startTime, params.timezone);
    const timeBucketAsLocal = startOfDay(interactionStartTimeAsLocal);
    const timeBucketAsUtc = startOfDay(parseISO(interaction.startTime));

    return {
        filter: {
            timeBucketAsLocal,
            timeBucketAsUtc,
            organisationId: params.organisationId,
            userId: params.userId,
            teamId: params.teamId,
        },
        update: {
            $set: {
                timeBucketAsLocal,
                timeBucketAsUtc,
                organisationId: params.organisationId,
                userId: params.userId,
                teamId: params.teamId,

                'interactions.$[interaction].workstreamDefinitionId': params.workstreamDefinitionId,
                'interactions.$[interaction].workstreamDefinitionName':
                    params.workstreamDefinitionName ?? `Unnamed Workstream (${params.workstreamDefinitionId})`,
                'interactions.$[interaction].taskTypeId': interaction.workloadTaskId,
                'interactions.$[interaction].startTime': interaction.startTime,
                'interactions.$[interaction].workstreamIdentifier': interaction.workstreamIdentifier,
                'interactions.$[interaction].isAssign': interaction.isAssign,
                'interactions.$[interaction].isPrivateComment': interaction.isPrivateComment,
                'interactions.$[interaction].isPublicComment': interaction.isPublicComment,
                'interactions.$[interaction].isReassign': interaction.isReassign,
                'interactions.$[interaction].statusChangeTo': interaction.statusChangeTo,
                'interactions.$[interaction].type': interaction.type,
                'interactions.$[interaction].workEffortSecs': interaction.workEffortSecs,
                'interactions.$[interaction].completionStatus': interaction.completionStatus,
                'interactions.$[interaction].talkTimeSecs': interaction.talkTimeSecs,
                'interactions.$[interaction].consultationTimeSecs': interaction.consultationTimeSecs,
                'interactions.$[interaction].wrapTimeSecs': interaction.wrapTimeSecs,
                'interactions.$[interaction].holdTimeSecs': interaction.holdTimeSecs,
                'interactions.$[interaction].waitTimeSecs': interaction.waitTimeSecs,
            },
        },

        arrayFilters: [{ 'interaction.identifier': interaction.identifier }],
    };
};
