import { log } from '../../../utils';
import type { APIGatewayProxyEvent } from 'aws-lambda';
import { endOfDay, startOfDay } from 'date-fns';
import { ReconciliationTypes, runAthenaQuery } from '../../../athena';
import { extractIdentity, formatAsJSend, JSend } from '@agyletime/wfm-aws-utils';
export const reconcileInteractionsHandler = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    log.debug('POST %s', event.path, event);

    const identity = extractIdentity(event);
    const organisationId = Number(identity.organisationId);

    const today = new Date();
    const fromDate = startOfDay(today);
    const toDate = endOfDay(today);

    const { queryExecutionId } = await runAthenaQuery(
        { organisationId, fromDate, toDate },
        ReconciliationTypes.Interaction,
    );

    return formatAsJSend(
        {
            message: `reconcileInteractions was called with orgId: ${organisationId}, fromDate: ${fromDate.toISOString()}, toDate: ${toDate.toISOString()}`,
            queryExecutionId: queryExecutionId,
        },
        200,
    );
};
