import { log } from '../../utils';
import { getTeamForUser } from '../../utils/getTeamForUser';
import { getTimezoneForUser } from '../../utils/getTimezoneForUser';
import calculateShiftCreatedDiff from './calculateShiftCreatedDiff';
import type { MongoRosteredMetric } from '@agyletime/performance-utils';
import type { MongoQueryWithFilter, ShiftCreatedEvent } from '../types';

export async function createShiftMetric(
    data: Omit<ShiftCreatedEvent, 'roster'>,
    agyletimeorganisationId: string | number,
): Promise<MongoQueryWithFilter<MongoRosteredMetric>[]> {
    const shift = data.shift;
    const shiftId = shift._id;

    log.debug(`createShiftMetric ${shift.user.team ? 'using team from shift' : 'no team on shift, looking up'}`);
    const team =
        shift.user.team ??
        (await getTeamForUser({
            organisationId: parseInt(agyletimeorganisationId.toString()),
            userId: shift.user.id,
        }));
    log.debug(
        `createShiftMetric ${shift.user.timezone ? 'using timezone from shift' : 'no timezone on shift, looking up'}`,
    );
    const timezone =
        shift.user.timezone ??
        (await getTimezoneForUser({
            organisationId: parseInt(agyletimeorganisationId.toString()),
            userId: shift.user.id,
        }));

    if (!team?.id || !agyletimeorganisationId || !shift.user.id) {
        log.error('Missing required data for event', {
            team,
            agyletimeorganisationId,
            user: shift.user.id,
            payload: data,
        });
        throw new Error('Missing required data for event');
    }

    const params = {
        organisationId: parseInt(agyletimeorganisationId.toString()),
        teamId: team.id,
        userId: shift.user.id,
        shiftId: shiftId,
        timezone,
    };

    const upserts = calculateShiftCreatedDiff({ newShiftEvent: data, params });
    log.debug('createShiftMetric Upserts created', { upserts: upserts.length });

    return upserts;
}
