import { log } from '../../utils';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils';

interface ShiftMetricWindowFilter {
    organisationId: number;
    startWindow: Date;
    endWindow: Date;
    byUtc: boolean;
}

export const flushShiftMetrics = async ({
    organisationId,
    startWindow,
    endWindow,
    byUtc = true,
}: ShiftMetricWindowFilter): Promise<void> => {
    if (!organisationId) {
        throw new Error('Cannot delete shift metrics without all unique identifier');
    }
    log.debug('flushShiftMetrics Deleting existing metrics', {
        organisationId,
        fromDate: startWindow,
        toDate: endWindow,
        filterOnUtc: true,
    });

    const dateField = byUtc ? 'timeBucketAsUtc' : 'timeBucketAsLocal';
    await MongoRosteredMetricModel.deleteMany({
        organisationId,
        [dateField]: {
            $gte: startWindow,
            $lt: endWindow,
        },
    });
};
