import type { DateRange } from '../../utils';
import { rebuildRosteredMetricsForOrg } from '../rebuildRosteredMetricsForOrg';
import type { OrgDetails, RebuildResults } from '../../types';
import { log } from '../../../utils';
import { getOrganisationApiParamsById, getSSMParameters } from '@agyletime/wfm-aws-utils';

export async function rebuildForSingleInterval({
    orgDetails,
    dateRanges,
}: {
    orgDetails: OrgDetails;
    dateRanges: DateRange[];
}): Promise<RebuildResults | undefined> {
    log.debug('rebuildForSingleInterval original dateRanges', { orgId: orgDetails.id, dateRanges });
    const dateRangeToProcess = dateRanges.pop();
    let rebuildResults: RebuildResults;
    if (dateRangeToProcess) {
        const organisationParameters = await getSSMParameters();

        const apiConnectionDetails = getOrganisationApiParamsById({
            organisationId: orgDetails.id,
            organisationParams: organisationParameters,
        });

        if (!apiConnectionDetails) throw new Error('Unable to load org params');

        const scheduleRebuildResults = await rebuildRosteredMetricsForOrg(
            orgDetails,
            dateRangeToProcess.fromDate,
            dateRangeToProcess.toDate,
            apiConnectionDetails,
        );

        // TODO: get this working again - ideally with data from the lake
        // const taskRebuildResults = await rebuildTaskDataMetricsForOrg(
        //     orgDetails,
        //     dateRangeToProcess.fromDate,
        //     dateRangeToProcess.toDate,
        // );

        rebuildResults = {
            error: scheduleRebuildResults.error // || taskRebuildResults.error
                ? [scheduleRebuildResults.error /*, taskRebuildResults.error */]
                : undefined,
            startWindow: scheduleRebuildResults.startWindow,
            endWindow: scheduleRebuildResults.endWindow,
            result: [scheduleRebuildResults.result /*, taskRebuildResults.result */],
        };

        log.debug('rebuildForSingleInterval new dateRanges', { orgId: orgDetails.id, dateRangeToProcess, dateRanges });
        return rebuildResults;
    }
    return {
        result: [],
        error: 'No date range to process',
    };
}
