import type { APIGatewayProxyEvent } from 'aws-lambda';
import { parseRequest } from './utils';
import { log } from '../../../utils';
import { NotAllowedError } from '../../../errors';
import { extractIdentity, formatAsJSend, invokeStepFunction, JSend } from '@agyletime/wfm-aws-utils';

export const rebuildRosteredMetricsHandler = async (event: APIGatewayProxyEvent): Promise<JSend> => {
    log.debug('POST %s', event.path, event);
    const identity = extractIdentity(event);
    // TODO: This should be replaced by the right scopes once implemented
    if (identity.identifier !== 'admin@agyletime.com') throw new NotAllowedError('Not allowed to perform this action');
    const { allOrgDetails, fromDate, toDate } = await parseRequest(event);

    const stepFunctionResults = [],
        stepFunctionErrors = [];
    for (const orgDetails of allOrgDetails) {
        try {
            const result = await invokeStepFunction({
                stateMachineArn: process.env['REBUILD_ROSTERED_METRICS_FOR_ORG_STATEMACHINE_ARN'] ?? '',
                payload: { orgDetails, fromDate, toDate, runImmediately: true },
            }); // See state-machines.yaml for step function definitions

            log.debug('Step function okay', { result });
            stepFunctionResults.push(result);
        } catch (e: unknown) {
            log.error('Step function fail', { e });
            stepFunctionErrors.push({ error: e });
        }
    }

    return formatAsJSend({ results: stepFunctionResults, errors: stepFunctionErrors }, 200);
};
