import { getRelevantOrgs } from './utils';
import type { ReconciliationScheduleJobEvent } from '../../types';
import { log } from '../../../utils';
import { invokeStepFunction } from '@agyletime/wfm-aws-utils';

export const reconcileRosteredMetricsHandler = async (event: ReconciliationScheduleJobEvent): Promise<void> => {
    log.debug('reconcileScheduleMetrics', event);
    const allOrgDetails = await getRelevantOrgs();
    const toDate = new Date();
    // Set toDate to nearest hour. If < 15 past, set to previous hour
    if (toDate.getMinutes() < 15) {
        toDate.setHours(toDate.getHours() - 1);
    }
    toDate.setMinutes(0);
    toDate.setSeconds(0);
    toDate.setMilliseconds(0);
    const fromDate = new Date(toDate);
    // Set fromDate to be `event.periodInHours` before toDate
    fromDate.setDate(toDate.getHours() - parseInt(event.periodInHours));

    const stepFunctionResults = [],
        stepFunctionErrors = [];
    for (const orgDetails of allOrgDetails) {
        try {
            const result = await invokeStepFunction({
                stateMachineArn: process.env['REBUILD_ROSTERED_METRICS_FOR_ORG_STATEMACHINE_ARN'] ?? '',
                payload: { orgDetails, fromDate, toDate },
            }); // See state-machines.yaml for step function definitions

            log.debug('Step function okay', { result });
            stepFunctionResults.push(result);
        } catch (e: unknown) {
            log.error('Step function fail', { e });
            stepFunctionErrors.push({ error: e });
        }
    }

    return;
};
