import type { RosterTaskDefinitionMongoSchema, TaskDefinitionMongoSchema } from '@agyletime/wfm-types';
import type { AgyleTimeAPIClientParams } from '@agyletime/wfm-types/lib/common/agyleTimeClient/agyleTimeClient';
import type { BulkWriteResult } from 'mongodb';

export interface TaskMergedTask {
    _id?: string;
    organisationId: number;
    identifier: string;
    agentAlias: string;
    createdAt: string;
    shiftId: string;
    startTime: string;
    endTime?: string | undefined;
    taskDefinitionId: number;
    taskTypeIdentifier: string;
    updatedAt: string;
    userEmail: string;
    userId: number;
}

export interface TaskDataMergedEvent {
    tasks: TaskMergedTask[];
    shiftId: string;
    userId: number;
}

export type TaskType = Omit<RosterTaskDefinitionMongoSchema, 'isShiftSwapAllowed' | 'categoryId' | 'categoryName'>;

export interface ShiftTask {
    _id: string;
    startTime: string;
    endTime: string;

    created?: string;
    modified?: string;

    policy?: object;

    taskType: TaskType;
}

export interface Shift {
    _id: string;
    changedByUser: object | { email: string; id: number };
    createdAt: string;
    endTime: string;
    hasOptInTask: boolean;
    leaveRequest: object;
    shiftTasks: ShiftTask[];
    shiftTemplate?: object;
    startTime: string;
    tags?: null[] | null;
    updatedAt: string;
    user: {
        email: string;
        id: number;
        timezone?: string;
        team?: {
            id: number;
            name: string;
        };
    };
    isAdditionalHour?: boolean;
}

export interface ShiftCreatedEvent {
    roster: {
        id: string;
        version: number;
    };
    shift: Shift;
}

export type ShiftUpdatedEvent = ShiftCreatedEvent & {
    previousShift: Shift;
};

export type ShiftDeletedEvent = ShiftCreatedEvent & {
    previousShift: Shift;
};

export interface MongoQueryWithFilter<T> {
    filter: { [K in keyof T]?: T[K] };
    query: {
        $set: { [K in keyof T]?: T[K] };
        $inc?: { [K in keyof T]?: T[K] };
    };
}
export interface MongoUnsetWithFilter<T> {
    filter: { [K in keyof T]?: T[K] };
    query: {
        $unset: { [K in keyof T]?: T[K] };
    };
}
export interface MongoDeleteWithFilter<T> {
    filter: { [K in keyof T]?: T[K] };
}

export interface ActualFilterParams {
    organisationId: number;
    userId: number;
    teamId?: number;
    shiftId: string;
    taskDefinitionsMap: Map<number, TaskDefinitionMongoSchema>;
    timezone: string;
}

export interface ShiftEventParams {
    organisationId: number;
    teamId: number;
    userId: number;
    shiftId: string;
    timezone: string;
}

export interface InteractionEventParams {
    organisationId: number;
    userId: number;
    teamId: number;
    interactionType: string;
    workstreamIdentifier: string;
    workstreamDefinitionId?: number | undefined;
    workstreamDefinitionName?: string | undefined;
    timezone: string;
    taskTypeId: number;
}

export interface WorkstreamMetricEventParams {
    organisationId: number;
    workstreamDefinitionId: number;
    workstreamDefinitionName?: string | undefined;
    workstreamIdentifier: string;
}

export interface ScheduleApiResponse {
    data: Shift[];
    nextPageToken: string | undefined;
}
export interface RebuildResult {
    error: unknown;
    result: BulkWriteResult | undefined;
    disabled?: boolean;
    startWindow?: Date | undefined;
    endWindow?: Date | undefined;
}
export interface RebuildResults {
    error: unknown;
    result: (BulkWriteResult | undefined)[];
    disabled?: boolean;
    startWindow?: Date | undefined;
    endWindow?: Date | undefined;
}

// Stripped down version of SSMParametersForOrganisation without sensitive data
export interface OrgDetails {
    id: number;
    name: string;
    api?: AgyleTimeAPIClientParams;
}

export interface ReconciliationScheduleJobEvent {
    periodInHours: `${number}`;
}

export interface Upsert<T> {
    filter: { [K in keyof T]?: T[K] };
    query: {
        $set?: Partial<T>; // May be set to undefined to remove data
        $inc?: Partial<T>;
    };
}

export interface Delete<T> {
    filter: { [K in keyof T]?: T[K] };
}

// eslint-disable-next-line @typescript-eslint/no-explicit-any
export interface MongoUpdateOneQuery<T extends MongoQueryWithFilter<any>> {
    updateOne: {
        filter: T['filter'];
        upsert: true;
        update: { $set?: T['query']['$set']; $inc?: T['query']['$inc'] };
    };
}

// eslint-disable-next-line @typescript-eslint/no-explicit-any
export interface MongoDeleteManyQuery<T extends MongoDeleteWithFilter<any>> {
    deleteMany: {
        filter: T['filter'];
    };
}

export interface RebuildMetricsPostBody {
    fromDate: string;
    toDate: string;
}
