import type { RebuildMetricsPostBody, ShiftTask, TaskMergedTask } from './types';
import { log } from '../utils';
import type { ISODateTimeString } from '@agyletime/wfm-types';
import { differenceInSeconds } from 'date-fns';
import type { PlayvoxWfmErrorOptions } from '@agyletime/wfm-aws-utils/lib/errors';
import { PlayvoxWfmError } from '@agyletime/wfm-aws-utils/lib/errors';

export const getDurationOfTaskMergedTask = (task: TaskMergedTask): number => {
    if (!task.endTime) return 0;

    const startTime = new Date(task.startTime);
    const endTime = new Date(task.endTime);

    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    return endTime ? differenceInSeconds(endTime, startTime) : 0;
};

export const getDurationOfShiftTask = (shiftTask: ShiftTask): number => {
    return shiftTask.endTime ? differenceInSeconds(new Date(shiftTask.endTime), new Date(shiftTask.startTime)) : 0;
};

export interface DateRange {
    fromDate: Date;
    toDate: Date;
}
export interface SerialisedDateRange {
    fromDate: ISODateTimeString;
    toDate: ISODateTimeString;
}
export const generateDailyIntervalsFromDateRange = ({ fromDate, toDate }: SerialisedDateRange): DateRange[] => {
    const dates = [];
    let currentDate = new Date(fromDate);
    const addDays = (currentDate: Date, days: number): Date => {
        const date = new Date(currentDate);
        date.setDate(date.getDate() + days);
        return date;
    };
    while (currentDate.toISOString() <= toDate) {
        const nextDate = addDays(currentDate, 1);
        dates.push({ fromDate: currentDate, toDate: nextDate });
        currentDate = nextDate;
    }
    log.info('generateDailyIntervalsFromDateRange', { fromDate, toDate, dates });
    return dates;
};

export const setUnnamedWorkstreamDefinitionName = ({ id }: { id: number }): string => `Unknown (id=${id})`;

export const isValidRebuildMetricsPostBody = (obj: unknown): obj is RebuildMetricsPostBody => {
    const body = obj as RebuildMetricsPostBody;
    return (
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.fromDate !== undefined &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.fromDate !== null &&
        body.fromDate !== '' &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.toDate !== undefined &&
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        body.toDate !== null &&
        body.toDate !== ''
    );
};

export const processEvents = async (handlers: Promise<unknown>[]): Promise<void> => {
    try {
        await Promise.all(handlers);
    } catch (error: unknown) {
        throw new PlayvoxWfmError((error as PlayvoxWfmError).message, { ...(error as PlayvoxWfmErrorOptions) });
    }
};

// A function which takes a date and return the latest between now and this date
export const getLatestBetweenDateAndNow = (input: string): Date => {
    const date = new Date(input);
    const now = new Date();
    return date.getTime() > now.getTime() ? date : now;
};
