import type { InteractionEventParams, MongoQueryWithFilter } from '../types';
import { config } from '../../config';
import { toZonedTime } from 'date-fns-tz';
import { parseISO } from 'date-fns';
import { roundDateToNearestTime } from '../../utils';
import { getHandleTime, type InteractionEventType } from '../interactions';
import type { MongoInteractionsWorkloadMetric } from '@agyletime/performance-utils';

export default ({
    interaction,
    params,
}: {
    interaction: InteractionEventType;
    params: InteractionEventParams;
}): MongoQueryWithFilter<MongoInteractionsWorkloadMetric> => {
    const interactionStartTimeAsLocal = toZonedTime(interaction.startTime, params.timezone);
    const timeBucketAsLocal = roundDateToNearestTime(interactionStartTimeAsLocal, config.mongoBucketSizeInMinutes);
    const timeBucketAsUtc = roundDateToNearestTime(parseISO(interaction.startTime), config.mongoBucketSizeInMinutes);

    const handleTime = getHandleTime(interaction);

    const incrementObject: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>['query']['$set'] = {
        totalHandleTime_interactions: handleTime,
    };

    const set: MongoQueryWithFilter<MongoInteractionsWorkloadMetric>['query']['$set'] = {
        organisationId: params.organisationId,
        timeBucketAsLocal,
        timeBucketAsUtc,
        userId: params.userId,
        teamId: params.teamId,
        workloadTaskId: params.taskTypeId,
    };

    return {
        filter: {
            organisationId: params.organisationId,
            timeBucketAsLocal,
            userId: params.userId,
            teamId: params.teamId,
            workloadTaskId: params.taskTypeId,
        },
        query: {
            $set: set,
            $inc: incrementObject,
        },
    };
};
