import type { MongoUnsetWithFilter } from '../types';
import type { MongoActualHoursWorkloadMetric } from '@agyletime/performance-utils';

interface ShiftMetricUniqueIdentifier {
    organisationId: number;
    userId: number;
    shiftId: string;
}

export const resetActualHoursWorkloadMetricsForShift = ({
    organisationId,
    userId,
    shiftId,
}: ShiftMetricUniqueIdentifier): MongoUnsetWithFilter<MongoActualHoursWorkloadMetric> => {
    if (!organisationId || !userId || !shiftId) {
        throw new Error('Cannot delete shift metrics without all unique identifier');
    }
    return {
        filter: {
            organisationId,
            shiftId,
            userId,
        },
        // Note that we don't unset shiftId (leave that for rostered hours)
        query: {
            $unset: {
                actualAvailableTime_actualHours: 1,
                actualAfterCallWorkTime_actualHours: 1,
                totalActualTasks_actualHours: 1,
            },
        },
    };
};
