import type { MongoRosteredHoursWorkloadMetric } from '@agyletime/performance-utils';
import type { MongoUnsetWithFilter } from '../types';

interface ShiftMetricUniqueIdentifier {
    organisationId: number;
    userId: number;
    shiftId: string;
}

export const resetRosteredHoursWorkloadMetricsForShift = ({
    organisationId,
    userId,
    shiftId,
}: ShiftMetricUniqueIdentifier): MongoUnsetWithFilter<MongoRosteredHoursWorkloadMetric> => {
    if (!organisationId || !userId || !shiftId) {
        throw new Error('Cannot delete shift metrics without all unique identifier');
    }
    return {
        filter: {
            organisationId,
            shiftId,
            userId,
        },
        query: {
            $unset: {
                shiftId: '',
                rosteredAvailableTime_rosteredHours: 1,
                totalRosteredTasks_rosteredHours: 1,
            },
        },
    };
};
