import type { WorkstreamMetricsEventType } from './workstreamMetricEvent.type';
import type { WorkstreamMetricEventParams } from '../types';
import { WorkstreamMetricEventParameterValidationError } from '../../errors';

export default ({ workstream }: { workstream: WorkstreamMetricsEventType }): WorkstreamMetricEventParams => {
    const organisationId = workstream.organisationId;
    if (!organisationId) throw new WorkstreamMetricEventParameterValidationError('Organisation ID not supplied');

    const workstreamIdentifier = workstream.workstreamIdentifier;
    if (!workstreamIdentifier)
        throw new WorkstreamMetricEventParameterValidationError('Workstream identifier not supplied');

    const workstreamDefinitionId = workstream.workstreamDefinitionId;
    const workstreamDefinitionName = workstream.workstreamDefinition?.name;

    return {
        organisationId,
        workstreamDefinitionId,
        workstreamIdentifier,
        workstreamDefinitionName,
    };
};
