import type { OptionalId } from 'mongodb';
import { ObjectId } from 'mongodb';

interface ControlLogModelConstructor {
    organisationId: string | number;
    successes: number;
    errors: number;
    jobStartedAt: Date;
    jobFinishedAt: Date;
    fetchStartedAt: Date;
    fetchFinishedAt: Date;
    totalItems: number;
}

export class ControlLogModel {
    _id?: ObjectId;
    organisationId: string | number;
    jobStartedAt: Date;
    jobFinishedAt: Date;
    fetchStartedAt: Date;
    fetchFinishedAt: Date;
    successes: number;
    errors: number;
    totalItems: number;

    constructor(params: ControlLogModelConstructor) {
        this.organisationId = params.organisationId;
        this.jobStartedAt = params.jobStartedAt;
        this.jobFinishedAt = params.jobFinishedAt;
        this.fetchStartedAt = params.fetchStartedAt;
        this.fetchFinishedAt = params.fetchFinishedAt;
        this.successes = params.successes;
        this.errors = params.errors;
        this.totalItems = params.totalItems;
    }

    toJSON(): {
        jobFinishedAt: Date;
        totalItems: number;
        organisationId: string | number;
        jobStartedAt: Date;
        successes: number;
        fetchFinishedAt: Date;
        errors: number;
        fetchStartedAt: Date;
    } {
        return {
            organisationId: this.organisationId,
            jobStartedAt: this.jobStartedAt,
            jobFinishedAt: this.jobFinishedAt,
            fetchStartedAt: this.fetchStartedAt,
            fetchFinishedAt: this.fetchFinishedAt,
            successes: this.successes,
            errors: this.errors,
            totalItems: this.totalItems,
        };
    }

    toDocument(): OptionalId<typeof ControlLogModel> {
        return { _id: new ObjectId(), ...this.toJSON() } as unknown as OptionalId<typeof ControlLogModel>;
    }
}
