import { log } from '../utils';
import { upsertMetricsConfiguration } from './upsertMetricsConfiguration';
import { APIGatewayEvent } from 'aws-lambda';
import { extractIdentity, formatAsJSend, JSend } from '@agyletime/wfm-aws-utils';

export const postMetricsConfiguration = async (event: APIGatewayEvent): Promise<JSend> => {
    try {
        log.debug('POST %s', event.path, event);
        const identity = extractIdentity(event);
        const organisationId = Number(identity.organisationId);

        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const metricsConfig = JSON.parse(event.body ? event.body : '[]');
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const response = await upsertMetricsConfiguration({ organisationId, metricsConfig });

        return formatAsJSend({ response }, 200);
    } catch (error: unknown) {
        log.warn('Failed Running POST %s', error);
        throw error;
    }
};
