import { createMongoConnection } from '../utils';
import type { MetricsConfigColumnDefinition } from '@agyletime/performance-utils';
import { UpdateResult } from 'mongodb';

interface Parameters {
    organisationId: string | number;
    metricsConfig?: MetricsConfigColumnDefinition[];
}

export const upsertMetricsConfiguration = async ({
    organisationId,
    metricsConfig,
}: Parameters): Promise<UpdateResult> => {
    const db = await createMongoConnection('performance');
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (!db?.database) {
        throw new Error('Failed to connect to MongoDB');
    }
    const filter = { organisationId: organisationId };
    const update = {
        $set: {
            organisationId: organisationId,
            metricsConfig: metricsConfig,
        },
    };
    const options = { upsert: true };

    return await db.collections.metrics_configuration.updateOne(filter, update, options);
};
