import { config } from '../../config';
import { init, log } from '../../utils';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import { getOrganisationApiParamsById, getSSMParameters } from '@agyletime/wfm-aws-utils';
import { TaskDefinitionApi, WfmApiClient } from '@agyletime/wfm-client';
import type { TaskDefinitionMongoSchema } from '@agyletime/wfm-types';
import { SQSRecord } from 'aws-lambda';

export const syncTaskDefinitionForOrganisation = async (record: SQSRecord): Promise<void> => {
    log.debug('syncTaskDefinitionForOrganisation - Received event', { record });
    await init(config.db);

    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const body: { agyletimeorganisationId: string } = JSON.parse(record.body);

    try {
        await init(config.db);

        const organisationParameters = await getSSMParameters();

        const apiConnectionDetails = getOrganisationApiParamsById({
            organisationId: parseInt(body.agyletimeorganisationId),
            organisationParams: organisationParameters,
        });

        if (!apiConnectionDetails) throw new Error('Unable to load org params');

        const apiConfig = {
            ...apiConnectionDetails,
            apiGatewayUrl: config.agyletime.api_gateway_base_url,
            organisationId: apiConnectionDetails.organisationId,
        };

        const agyleTimeClient = new WfmApiClient(apiConfig);

        const definitionApi = new TaskDefinitionApi(agyleTimeClient);

        const definitionData = await definitionApi.getAllTaskDefinitions();

        const definitions = definitionData.data as TaskDefinitionMongoSchema[];

        log.debug('definitions', { definitionData });

        await TaskDefinitionModel.deleteMany({
            organisationId: parseInt(body.agyletimeorganisationId),
        });
        await TaskDefinitionModel.insertMany(
            definitions.map((definition) => ({
                ...definition,
                organisationId: body.agyletimeorganisationId,
                adherenceEquivalentTasks:
                    definition.adherenceEquivalentTasks && definition.adherenceEquivalentTasks instanceof Array
                        ? definition.adherenceEquivalentTasks
                        : null,
            })),
        );
    } catch (error: unknown) {
        log.error('Failed to generate due to error: ', { error });
        throw error;
    }
};
