import { log } from '../../utils';
import { formatAsJSend, getSSMParameters, JSend, SSMParameters } from '@agyletime/wfm-aws-utils';
import { invokeTaskDefinitionSyncJob } from '../invokeTaskDataSyncJob';
import { APIGatewayEvent } from 'aws-lambda';

let organisationParameters: SSMParameters = [];

/**
 * This is a lambda to sync task definitions to workactivity mongo cluster in batches
 *
 * Sample payload if you need to trigger this lambda manually to sync a specific organisationId
 * 		{ "organisationId": 999 }
 * @param event
 */
export const handleTaskDefinitionSync = async (event: APIGatewayEvent): Promise<JSend> => {
    log.info(`taskDefinitionSync`, { event });
    const organisationIdsString = event.queryStringParameters?.['organisationIds'];
    const organisationIds = organisationIdsString ? organisationIdsString.split(',').map((id) => parseInt(id)) : [];

    try {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!organisationParameters?.length) {
            log.debug('Fetching getSSMParameters');
            organisationParameters = await getSSMParameters();
            log.debug('Got parameters', { organisationParameters });
        }

        for (const apiConn of organisationParameters) {
            if (organisationIds.length === 0 || organisationIds.includes(parseInt(apiConn.id.toString()))) {
                log.debug('Running sync for org', apiConn);
                await invokeTaskDefinitionSyncJob({
                    organisation: apiConn,
                    organisationId: apiConn.id,
                    customerName: apiConn.name,
                });
            }
        }

        return formatAsJSend({ message: 'Complete' });
    } catch (error: unknown) {
        log.error('Failed handleUserSyncs', { error });
        return formatAsJSend({ error });
    }
};
