import { config } from '../config';
import { log } from '../utils';
import { SSMParametersForOrganisation, writeToQueue } from '@agyletime/wfm-aws-utils';
import { formatEventAsCloudEvent } from '@agyletime/wfm-aws-utils';
const queueUrl = config.performance.syncTaskDefinitionQueueUrl;

interface JobParams {
    organisation: SSMParametersForOrganisation;
    organisationId: number;
    customerName: string;
}

/**
 * Invoke daily forecast request job which will be picked up by the serverless forecasting algorithm
 *
 * @param job
 * @returns {Promise<void>}
 */
export const invokeTaskDefinitionSyncJob = async (job: JobParams): Promise<void> => {
    const payLoadData = {
        organisationId: job.organisationId,
        customerName: job.customerName,
    };

    const eventType = 'task.definition.sync';
    const payLoad = formatEventAsCloudEvent({
        organisationId: job.organisationId,
        eventType,
        eventSourceUri: '/performance/task-definition-sync',
        data: payLoadData,
    });
    const cloudEventId = payLoad.id;

    log.debug(`Sending SQS Msg to task definition sync `, {
        payLoad: payLoad,
        queueUrl: queueUrl,
    });

    if (!queueUrl) throw new Error('queueUrl not defined');

    // eslint-disable-next-line @typescript-eslint/no-confusing-void-expression
    const sentResult = await writeToQueue({ cloudEvent: payLoad, queueUrl });

    log.debug(`Message sent to sqs`, { cloudEventId, sentResult });
};
