import { TaskDefinitionModel } from '@agyletime/performance-utils';

export interface UpdateLastOccuranceParams {
    organisationId: number;
    taskId: number;
    date?: Date;
}

export const updateLastOccurance = async (params: Map<number, UpdateLastOccuranceParams>): Promise<void> => {
    const now = new Date();
    const updates = Array.from(params.values()).map(({ organisationId, taskId, date = now }) => ({
        updateOne: {
            filter: { organisationId, id: taskId },
            update: [
                {
                    $set: {
                        lastOccurance: {
                            // Only update if new date is greater than old one
                            $cond: [
                                {
                                    $gt: [date, '$lastOccurance'],
                                },
                                date,
                                {
                                    $ifNull: ['$lastOccurance', now], // Set to current date if lastOccurance is null
                                },
                            ],
                        },
                    },
                },
            ],
        },
    }));

    await TaskDefinitionModel.bulkWrite(updates);
};
