import type { InferSchemaType } from 'mongoose';
import mongoose, { Schema } from 'mongoose';

const schema = new Schema(
    {
        organisationId: { type: Number, required: true },
        id: { type: Number, required: true },
        team: {
            _id: false,
            id: { type: Number },
            name: { type: String },
        },
        firstName: { type: String, required: false },
        lastName: { type: String, required: false },
        timezone: { type: String, required: false },
        tags: [
            {
                _id: false,
                text: { type: String },
            },
        ],
        roles: [
            {
                _id: false,
                id: { type: Number },
                name: { type: String },
            },
        ],
        email: { type: String, required: true },
        integrations: [
            {
                _id: false,
                integrationId: { type: Number },
                integrationName: { type: String },
                integrationAlias: { type: String },
                configuration: { type: String },
            },
        ],
    },
    {
        collection: 'users_cache',
    },
);

schema.index({
    organisationId: 1,
    userId: 1,
});
schema.index({
    organisationId: 1,
    'roles.id': 1,
});

schema.index({
    organisationId: 1,
    'team.id': 1,
});

schema.index({
    organisationId: 1,
    'tags.text': 1,
});

schema.index({
    organisationId: 1,
    'integrations.integrationAlias': 1,
});

schema.index(
    {
        organisationId: 1,
        id: 1,
    },
    { unique: true },
);
export type UserCacheType = InferSchemaType<typeof schema>;
export const UserCacheModel = mongoose.model<UserCacheType>('UserCacheModel', schema);
