import { config } from '../config';
import * as mongodb from '../utils/mongodb';
import { log } from '../utils';
import { fetchUserById } from './utils/fetchUserById';
import { insertOrUpdateUsers } from './utils/insertOrUpdateUsers';
import { getOrganisationApiParamsById, getSSMParameters } from '@agyletime/wfm-aws-utils';
import type { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';

export const doHandleInboundUserEvent = async (
    eventType: string,
    organisationId: number,
    eventData: unknown,
): Promise<void> => {
    // Sync created or update user event to mongo db
    // @ts-expect-error unsafe use of any
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access,@typescript-eslint/no-unsafe-assignment
    const userId: number = eventData.user.id;
    log.debug(`Syncing orgId: ${organisationId} userId: ${userId}`, { eventType, organisationId, userId });
    await mongodb.init(config.db);

    const organisationParameters = await getSSMParameters();

    const apiConnectionDetails = getOrganisationApiParamsById({
        organisationId: organisationId,
        organisationParams: organisationParameters,
    });

    if (!apiConnectionDetails) throw new Error('Unable to load org params');

    const user = await fetchUserById(organisationId, userId, apiConnectionDetails);
    // { team: 1, roles: 1, tags: 1, firstName: 1, lastName: 1, email: 1, timezone: 1, id: 1}
    const performanceUser = {
        team: user.team,
        roles: user.roles,
        tags: user.tags,
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        firstName: user.firstName ?? '',
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        lastName: user.lastName ?? '',
        email: user.email,
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        timezone: user.timezone ?? '',
        id: user.id,
        organisationId: organisationId,
        integrations: user.integrations,
    } as UserMongoSchema;
    await insertOrUpdateUsers({ organisationId, users: [performanceUser] });
    log.info(`OrgId [${organisationId}] User Id [${userId}] is synced`);
};
