import { config } from '../config';
import { log } from '../utils';
import { filterOrgParams } from './utils/filterOrgParams';
import { syncUsersToDb } from './syncUsersToDb';
import * as mongodb from '../utils/mongodb';
import type { SSMParameters } from '@agyletime/wfm-aws-utils';

/**
 * Sync organisation users in batches
 * Currently it's configured to sync in 4 slots/batches
 *
 * i.e
 *  - orgId `1,5,9,13,17,21,25,29,33....`  will be synced in batch 1 (run at UTC hour `1,5,9,13,17,21`)
 *  - orgId `2,6,10,14,18,22,26,30,34....` will be synced in batch 2 (run at UTC hour `2,6,10,14,18,22`)
 *  - orgId `3,7,11,15,19,23,27,31,35....` will be synced in batch 3 (run at UTC hour `3,7,11,15,19,23`)
 *  - orgId `4,8,12,16,20,24,28,32,36....` will be synced in batch 4 (run at UTC hour `0,4,8,12,16,20`)
 * @param organisationParameters
 * @param orgIdToSync
 */
export const doHandleUserSync = async (organisationParameters: SSMParameters, orgIdToSync: number): Promise<void> => {
    const filteredOrgParams = filterOrgParams(organisationParameters, orgIdToSync);
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (!filteredOrgParams?.length) {
        return;
    } // nothing to sync

    // this lambda will be triggered hourly via schedule job.
    await mongodb.init(config.db);

    // Sync all tenants in sequential
    for (const organisation of filteredOrgParams) {
        const { id: organisationId } = organisation;
        log.info(`Syncing orgId: ${organisationId}`, { organisation });
        await syncUsersToDb(organisation);
    }
};
