import { doHandleInboundUserEvent } from '../doHandleInboundUserEvent';
import { log } from '../../utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { KinesisStreamEvent } from 'aws-lambda';

export const handleInboundUsersEvent = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    log.debug(`received kinesis event with ${eventBatch.Records.length} records`);

    // @ts-expect-error implicit any
    const promises = [],
        events = [];

    eventBatch.Records.forEach((record) => {
        // TODO: get a proper type
        const plvxEvent: {
            id: string;
            agyletimeorganisationId: number;
            type: string;
            data: unknown;
        } = decodeKinesisPayload({ data: record.kinesis.data });

        log.debug(`event payload for event id ${plvxEvent.id}`, { plvxEvent });
        events.push(plvxEvent);

        const organisationId: number = plvxEvent.agyletimeorganisationId;

        const eventType: string = plvxEvent.type,
            eventData = plvxEvent.data;

        promises.push(doHandleInboundUserEvent(eventType, organisationId, eventData));
    });

    // @ts-expect-error Promises are explicitly any
    const resolutions = await Promise.allSettled(promises);
    const failed = resolutions.filter((item) => item.status !== 'fulfilled');

    log.info(`Completed. ${failed.length} failed from ${promises.length} records`);

    // eslint-disable-next-line @typescript-eslint/prefer-for-of
    for (let i = 0; i < resolutions.length; i++) {
        if (resolutions[i]?.status !== 'fulfilled') {
            // @ts-expect-error reason does not exist on any
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            log.error(`Failed reason: ${resolutions[i]?.reason}`, { error: resolutions[i]?.reason });
        }
    }
    return;
};
