import { doHandleUserSync } from '../doHandleUserSync';
import { log } from '../../utils';
import { getSSMParameters, type SSMParameters } from '@agyletime/wfm-aws-utils';

let organisationParameters: SSMParameters = [];

/**
 * This is a schedule lambda to sync users to workactivity mongo cluster in batches
 *
 * Sample payload if you need to trigger this lambda manually to sync a specific organisationId
 * 		{ "organisationId": 999 }
 * @param event
 */
export const handleUserSync = async (event: { organisationId?: number }): Promise<string> => {
    log.info(`handleUserSyncs`, { event });
    // this field will only be available if lambda is triggered manually with the payload { "organisationId": 999 }
    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
    const orgIdToSync: number = event.organisationId || 0;
    try {
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!organisationParameters?.length) {
            log.debug('Fetching getSSMParameters');
            organisationParameters = await getSSMParameters();
            log.debug('Got parameters', { organisationParameters });
        }
        await doHandleUserSync(organisationParameters, orgIdToSync);
    } catch (error: unknown) {
        log.error('Failed handleUserSyncs', { error });
    }
    return 'Completed';
};
