import { config } from '../../config';
import type { OrganisationId } from '@agyletime/wfm-types';
import { isError, UserApi, WfmApiClient } from '@agyletime/wfm-client';
import { BasicAuthApiClientParams } from '@agyletime/wfm-client/lib/types/agyleTimeClient/agyleTimeClient';
import type { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';

/**
 * Fetch Users by orgId (Only fetch these fields `id, firstName, lastName, email, team, roles & tags`)
 *
 * Set the page size to 10K since the user payload is very small
 *
 * @param organisationId
 * @param apiConnectionDetails
 */
export const fetchAllUsers = async (
    organisationId: OrganisationId,
    apiConnectionDetails: BasicAuthApiClientParams,
): Promise<UserMongoSchema[]> => {
    // this api config does NOT contain apiConnectionDetails since it's not required to hit ECS url
    const apiConfig = {
        ...apiConnectionDetails,
        apiGatewayUrl: config.agyletime.api_gateway_base_url,
        organisationId: organisationId,
    };
    const agyleTimeClient = new WfmApiClient(apiConfig);
    const userApi = new UserApi(agyleTimeClient);

    // only fetch the below fields in user object
    const options = {
        fields: {
            team: true,
            roles: true,
            tags: true,
            firstName: true,
            lastName: true,
            email: true,
            timezone: true,
            id: true,
            integrations: true,
            _id: false,
        },
        pageSize: 10000,
    };

    const response = await userApi.getAllUsers(true, {}, options);

    if (isError(response)) throw new Error(`Error calling getAllUsers: ${response.message}`);

    return response.data;
};
