import { config } from '../../config';
import type { OrganisationId } from '@agyletime/wfm-types';
import { isError, UserApi, WfmApiClient } from '@agyletime/wfm-client';
import type { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';
import type { BasicAuthApiClientParams } from '@agyletime/wfm-client/lib/types/agyleTimeClient/agyleTimeClient';

export const fetchUserById = async (
    organisationId: OrganisationId,
    userId: number,
    apiConnectionDetails: BasicAuthApiClientParams,
): Promise<UserMongoSchema> => {
    const apiConfig = {
        ...apiConnectionDetails,
        apiGatewayUrl: config.agyletime.api_gateway_base_url,
        organisationId: organisationId,
    };
    const agyleTimeClient = new WfmApiClient(apiConfig);
    const userApi = new UserApi(agyleTimeClient);
    const response = await userApi.getUserById(userId, null);

    if (isError(response)) throw new Error(`Error retrieving user: ${response.message}`);

    return response.data;
};
