import { log } from '../../utils';
import moment from 'moment';
import type { SSMParameters, SSMParametersForOrganisation } from '@agyletime/wfm-aws-utils';

/**
 * if orgIdToSync is available, this means the lambda is triggered adhoc for a specific org.
 * otherwise this will be invoked via scheduled cloudwatch event.
 * @param organisationParameters
 * @param orgIdToSync
 */
export const filterOrgParams = (organisationParameters: SSMParameters, orgIdToSync: number): SSMParameters => {
    const filteredOrgParameters: SSMParameters = [];
    if (orgIdToSync) {
        const organisation = organisationParameters.find(
            (organisation: SSMParametersForOrganisation) => organisation.id === orgIdToSync,
        );
        if (organisation) {
            log.info(`Adding Adhoc organisationId [${orgIdToSync}] for sync`, { organisation });
            filteredOrgParameters.push(organisation);
        } else {
            log.warn(`Adhoc organisationId [${orgIdToSync}] is not found in SSM Dynamo`);
        }
    } else {
        const day = getDay();
        organisationParameters.forEach((organisation: SSMParametersForOrganisation) => {
            const { id: organisationId } = organisation;
            if (shouldRunSyncForOrgId(organisationId, day)) {
                log.info(`Adding organisationId [${organisationId}] to batch [${day % 2}]`, { organisation });
                filteredOrgParameters.push(organisation);
            }
        });
    }
    return filteredOrgParameters;
};

// The goal here is to split it in to two ROUGHLY, odd numbers will go on odd days and even numbers on even days.
export const shouldRunSyncForOrgId = (orgId: number, day: number): boolean => orgId % 2 === day % 2;

export const getDay = (): number => moment().utcOffset(0).day();
