import { UserCacheModel } from '../UserCacheModel';
import { log } from '../../utils';
import { UserMongoSchema } from '@agyletime/wfm-types/lib/organisation/user';

interface InsertOrUpdateUsersParams {
    organisationId: number;
    users: UserMongoSchema[];
}
export const insertOrUpdateUsers = async ({ organisationId, users }: InsertOrUpdateUsersParams): Promise<void> => {
    const updates = [];
    for (const user of users) {
        const query = { $and: [] };
        // @ts-expect-error type mismatch
        query.$and.push({ organisationId });
        // @ts-expect-error type mismatch
        query.$and.push({ id: user.id });

        const updateLogic = {
            $set: {
                ...user,
            },
        };
        updates.push({
            updateOne: {
                filter: { ...query },
                update: { ...updateLogic },
                upsert: true,
            },
        });
    }
    try {
        await UserCacheModel.bulkWrite(updates);
    } catch (error: unknown) {
        log.error('Error in insertOrUpdateUsers', { organisationId, error, users });
        console.error(error);
        throw error;
    }
};
