import type { InferSchemaType } from 'mongoose';
import mongoose, { Schema } from 'mongoose';
import type { UserCacheType } from '../userCache/UserCacheModel';
import type { UserMongoSchema } from '@agyletime/wfm-types';

const schema = new Schema(
    {
        organisationId: { type: Number, required: true },
        updatedAt: { type: Date, required: true },
        userId: { type: Number, required: true },
        teamId: { type: Number, required: false },
        timezone: { type: String, required: false },
    },
    {
        collection: 'user_team_history',
    },
);

schema.index({
    organisationId: 1,
    userId: 1,
});

export type UserHistory = InferSchemaType<typeof schema>;
export const UserHistoryModel = mongoose.model<UserHistory>('UserHistoryModel', schema);

export interface AggregatedUserHistory {
    organisationId: number;
    userId: number;
    teamId: number;
    timezone: string;
}

export type UserWithUpdatedAt = (UserCacheType | UserMongoSchema) & { updatedAt: Date };
