import { log } from '../../utils';
import { config } from '../../config';
import * as mongodb from '../../utils/mongodb';
import type { UserMongoSchema, UserUpdatedEvent } from '@agyletime/wfm-types';
import type { AgyleTimeCloudEvent } from '@agyletime/wfm-aws-utils';
import { decodeKinesisPayload } from '@agyletime/wfm-aws-utils';
import { fromUnixTime } from 'date-fns';
import UserHistoryProvider from '../UserHistoryProvider';
import type { KinesisStreamEvent } from 'aws-lambda';
import { BadRequestError } from '../../errors';

export const userHistoryHandler = async (eventBatch: KinesisStreamEvent): Promise<void> => {
    await mongodb.init(config.db);

    const users: (UserMongoSchema & { updatedAt: Date })[] = [];
    const errors: Error[] = [];

    for (const record of eventBatch.Records) {
        try {
            log.debug('UserUpdated event ', { event: record });
            await mongodb.init(config.db);
            const event = decodeKinesisPayload<AgyleTimeCloudEvent<UserUpdatedEvent>>({ data: record.kinesis.data });
            log.debug('UserUpdated decodedPayload', { decodedPayload: event });
            const { user } = event.data;

            const userWithUpdatedDate = {
                ...user,
                updatedAt: fromUnixTime(record.kinesis.approximateArrivalTimestamp),
            };

            users.push(userWithUpdatedDate);
        } catch (e: unknown) {
            errors.push(e as Error);
        }
    }

    await UserHistoryProvider.saveUserHistoryFromEvent(users);

    if (errors.length > 0)
        throw new BadRequestError(`Error processing batch: ${errors.map((e) => e.message).toString()}`);
};
