import type { MongoQueryWithFilter, MongoUnsetWithFilter } from '../metrics';
import type { AnyBulkWriteOperation } from 'mongoose';
import { type UpdateOneModel } from 'mongoose';

interface ConvertUpsertsToBulkWriteOptions {
    isUpsert: boolean;
    updateMany?: boolean;
}
export const convertUpsertsToBulkWrite = <T>(
    upserts: (MongoQueryWithFilter<T> | MongoUnsetWithFilter<T>)[],
    { isUpsert, updateMany }: ConvertUpsertsToBulkWriteOptions = { isUpsert: true, updateMany: false },
): AnyBulkWriteOperation[] => {
    return upserts.map((upsert) => {
        return {
            ...(!updateMany && {
                updateOne: {
                    filter: upsert.filter,
                    upsert: isUpsert,
                    update: upsert.query,
                },
            }),
            ...(updateMany && {
                updateMany: {
                    filter: upsert.filter,
                    upsert: isUpsert,
                    update: upsert.query,
                },
            }),
        } as AnyBulkWriteOperation;
    });
};

export const convertV2UpdateToBulkWrite = <T>(updates: UpdateOneModel<T>[]): AnyBulkWriteOperation[] => {
    return updates.map((update) => {
        return {
            updateOne: {
                filter: update.filter,
                update: update.update,
                ...(update.upsert && { upsert: update.upsert }),
                ...(update.arrayFilters && { arrayFilters: update.arrayFilters }),
            },
        };
    });
};
