import { config } from '../config';
import { logger } from '@agyletime/common-utils';
const log = logger(config.log);

import type { Mongoose } from 'mongoose';
import { connect, connection } from 'mongoose';

interface MongoDbInitParams {
    url: string;
    name: string;
}

let connectionObject: Mongoose;
let initLog: boolean;
export const init = async ({ url, name }: MongoDbInitParams): Promise<Mongoose> => {
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
    if (connectionObject) {
        log.debug('Connection exist');
        return connectionObject;
    }

    if (!initLog) {
        connection.on('error', function (err) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            log.error('Mongoose err listener', { err });
            console.error(err);
        });

        connection.on('disconnected', function (err) {
            if (err) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                log.warn('Mongoose disconnected', { err });
                console.error(err);
            } else {
                log.debug('Mongoose disconnected');
            }
        });
        connection.on('open', function (err) {
            if (err) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                log.warn('Mongoose connection opened', { err });
                console.error(err);
            } else {
                log.debug('Mongoose connection opened');
            }
        });
    }
    const appName = process.env['AWS_LAMBDA_FUNCTION_NAME'] ?? '';
    connectionObject = await connect(url, {
        maxPoolSize: 10, // default, maxPoolSize is 100 in mongoose 7 doco
        serverSelectionTimeoutMS: 30000, // Keep trying to send operations for 30 seconds
        socketTimeoutMS: 2 * 60000, // Close sockets after 2 minutes of inactivity
        dbName: name,
        appName,
    });
    return connectionObject;
};

export const closeConnection = async (): Promise<void> => {
    await connection.close();
};
