import { getMongoDBBuilder } from '@agyletime/common-utils/mongo';
import { ControlLogModel } from '../models';
import { log } from './log';
import { config } from '../config';
import { IntermediateWorkstreamMetricsModel, MetricsConfigurationModel } from '@agyletime/performance-utils';
import type { Db } from 'mongodb';

const collections = {
    agent_workstream_metrics: IntermediateWorkstreamMetricsModel,
    agent_workstream_control_log: ControlLogModel,
    metrics_configuration: MetricsConfigurationModel,
} as const;

export const createMongoConnection = getMongoDBBuilder({ url: config.db.url, collections, logger: log });

export const validateDbConnection = <T extends { database?: Db }>(db: T): T & { database: Db } => {
    if (!db.database) {
        throw new Error('Failed to connect to MongoDB');
    }

    return db as T & { database: Db };
};
