import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import { ObjectId } from 'mongodb';
import upsertWorkstreamDefinitions from '../upsertWorkstreamDefinitions';
import { setUnnamedWorkstreamDefinitionName } from '../../metrics';
import { config } from '../../config';
import * as mongodb from '../../utils/mongodb';
import { log } from '../../utils';

const limit = Number(process.env['WORKSTREAM_DEV_MIGRATION_PAGE_SIZE']);
export const createWsDefFromPerformanceMetrics = async ({
    nextPageToken,
}: {
    nextPageToken: ObjectId;
}): Promise<ObjectId | undefined> => {
    await mongodb.init(config.db);
    const performanceMetrics = await MongoInteractionMetricModel.find({
        workstreamDefinitionId: { $exists: true },
        $or: [{ utc: { $exists: true } }, { year: { $exists: true } }],
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        ...(nextPageToken && { _id: { $gt: new ObjectId(nextPageToken) } }),
    })
        .sort({ year: 1, month: 1, day: 1, hour: 1, minute: 1 })
        .limit(limit)
        .lean();

    log.info(`Found ${performanceMetrics.length} metrics.`);
    // Get Workstream Definitions from metrics
    const workstreamDefinitions = performanceMetrics
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        .filter((m) => m.workstreamDefinitionId && (m.timeBucketAsUtc || m.timeBucketAsLocal)) // Just to be safe - but this should be covered in the mongo query
        .map((m) => {
            // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
            const metricDate = m.timeBucketAsUtc ?? m.timeBucketAsLocal;
            return {
                // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
                id: m.workstreamDefinitionId || 0, // It should never get to 0 because of filter above, but linter can't work this out
                name:
                    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
                    m.workstreamDefinitionName ||
                    // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
                    setUnnamedWorkstreamDefinitionName({ id: m.workstreamDefinitionId || 0 }),
                organisationId: m.organisationId,
                lastOccurance: metricDate,
                createdDate: metricDate,
            };
        });
    log.info(`Mapped into ${workstreamDefinitions.length} workstream definitions.`);
    // Rebuild
    await upsertWorkstreamDefinitions(workstreamDefinitions);
    log.info(`Upserted ${workstreamDefinitions.length} workstream definitions.`);
    return performanceMetrics[performanceMetrics.length - 1]?._id;
};
