import type { WorkstreamDefinitionsType } from './WorkstreamDefinitionsModel';
import { WorkstreamDefinitionsModel } from './WorkstreamDefinitionsModel';

export default async (workstreamDefinitions: WorkstreamDefinitionsType[]): Promise<void> => {
    const now = new Date();
    await WorkstreamDefinitionsModel.bulkWrite(
        workstreamDefinitions.map((workstreamDefinition) => {
            const lastOccurance = workstreamDefinition.lastOccurance ?? now;
            const createdDate = workstreamDefinition.createdDate ?? now;
            return {
                updateOne: {
                    filter: {
                        organisationId: workstreamDefinition.organisationId,
                        id: workstreamDefinition.id,
                        name: workstreamDefinition.name,
                    },
                    update: [
                        {
                            $set: {
                                ...workstreamDefinition,
                                createdDate: {
                                    $cond: [
                                        {
                                            $or: [
                                                { $not: ['$createdDate'] }, // checks if 'createdDate' does not exist
                                                { $lt: [createdDate, '$createdDate'] }, // or is greater (later) than 'workstreamDefinition.createdDate'
                                            ],
                                        },
                                        createdDate,
                                        '$createdDate',
                                    ],
                                },
                                lastOccurance: {
                                    $cond: [
                                        { $gt: [lastOccurance, '$lastOccurance'] },
                                        lastOccurance,
                                        '$lastOccurance',
                                    ],
                                },
                                occuranceCount: {
                                    $cond: [{ $not: ['$occuranceCount'] }, 1, { $add: ['$occuranceCount', 1] }],
                                },
                            },
                        },
                    ],
                    upsert: true,
                },
            };
        }),
    );
};
