import { MongoActualMetricModel } from '@agyletime/performance-utils';
import * as deleteActualsFromMongo from '../../../lib/athena/actuals/deleteActualsFromMongo';
import { config } from '../../../lib/config';
import { handleTaskDataQuery } from '../../../lib/athena/handlers/handleTaskDataQuery';
import { UserCacheModel } from '../../../lib/userCache/UserCacheModel';
import { TaskDefinitionModel } from '@agyletime/performance-utils';
import { ReconciliationTypes } from '../../../lib/athena';

jest.mock('../../../lib/userCache/UserCacheModel');
jest.createMockFromModule('@agyletime/performance-utils');

let mockUserCacheAggregate: jest.SpyInstance;
let mockTaskDefinitionFind: jest.SpyInstance;
let mockDeleteActualsFromMongo: jest.SpyInstance;
let mockMongoActualMetricModel: jest.SpyInstance;

describe('handleTaskDataQuery', () => {
    beforeEach(() => {
        mockUserCacheAggregate = jest.spyOn(UserCacheModel, 'aggregate').mockResolvedValue([
            {
                id: 1,
                organisationId: 1,
                firstName: 'first',
                lastName: 'last',
                timezone: 'A',
                team: {
                    id: 1,
                },
            },
        ]);
        // @ts-expect-error - not sure why this errors, but it works okay in runtime. Fix later.
        mockTaskDefinitionFind = jest.spyOn(TaskDefinitionModel, 'find').mockImplementation(() => ({
            lean: () =>
                Promise.resolve([
                    {
                        oragnisationId: 1,
                        id: 1,
                    },
                ]),
        }));
        mockDeleteActualsFromMongo = jest.spyOn(deleteActualsFromMongo, 'deleteActualsFromMongo').mockImplementation();
        mockMongoActualMetricModel = jest.spyOn(MongoActualMetricModel, 'bulkWrite').mockImplementation();
        config.featureFlags.orgAllowListTPM = '*';
    });
    it('should expose a function', () => {
        expect(handleTaskDataQuery).toBeDefined();
    });

    it('handleTaskDataQuery should return expected output', async () => {
        const results = {
            Items: [
                {
                    organisation_id: BigInt(1),
                    identifier: '1',
                    agentalias: '1',
                    createdat: '1',
                    shiftid: '1',
                    starttime: new Date().toISOString(),
                    endtime: new Date().toISOString(),
                    taskdefinitionid: 1,
                    tasktypeidentifier: '1',
                    updatedat: '1',
                    useremail: '1',
                    userid: BigInt(1),
                },
            ],
            QueryExecutionId: '1',
        };
        const queryMetadata = {
            queryExecutionId: '12345',
            organisationId: 1,
            fromDate: new Date(),
            toDate: new Date(),
            metricType: ReconciliationTypes.TaskData,
        };
        const event = {
            Records: [
                {
                    eventVersion: '1',
                    eventSource: '1',
                    awsRegion: '1',
                    eventTime: '1',
                    eventName: '1',
                    userIdentity: {
                        principalId: '1',
                    },
                    requestParameters: {
                        sourceIPAddress: '1',
                    },
                    responseElements: {
                        'x-amz-request-id': '1',
                        'x-amz-id-2': '1',
                    },
                    s3: {
                        s3SchemaVersion: '1',
                        configurationId: '1',
                        bucket: {
                            name: '1',
                            ownerIdentity: {
                                principalId: '1',
                            },
                            arn: '1',
                        },
                        object: {
                            key: '1',
                            size: 1,
                            eTag: '1',
                            versionId: '1',
                            sequencer: '1',
                        },
                    },
                },
            ],
        };
        let testError;

        try {
            await handleTaskDataQuery(results, queryMetadata, event);
        } catch (error: unknown) {
            testError = error;
        }
        expect(testError).toBeUndefined();
        expect(mockUserCacheAggregate).toHaveBeenCalled();
        expect(mockTaskDefinitionFind).toHaveBeenCalled();
        expect(mockDeleteActualsFromMongo).toHaveBeenCalled();
        expect(mockMongoActualMetricModel).toHaveBeenCalledWith([
            {
                replaceOne: {
                    filter: {
                        organisationId: 1,
                        shiftId: '1',
                        teamId: 1,
                        userId: 1,
                    },
                    upsert: true,
                    replacement: {
                        organisationId: 1,
                        shiftId: '1',
                        teamId: 1,
                        userId: 1,
                        shiftTasks: [],
                    },
                },
            },
        ]);
    });
});
