import { MongoMemoryServer } from 'mongodb-memory-server';
import { config } from '../../lib/config';
import mongoose from 'mongoose';

export = async function globalSetup() {
    if (process.env['SKIP_INTEGRATION_TESTS']) return;
    const domainAndPort = config.memoryServer.url.split('://')[1];
    if (!domainAndPort) throw new Error('Config is invalid');

    const domain = domainAndPort.split(':')[0];
    const port = domainAndPort.split(':')[1];

    if (!domain || !port) {
        throw new Error('Invalid params for Mongo Memory Server');
    }

    // @ts-expect-error global has no type
    (global as unknown).__MONGOINSTANCE__ = await MongoMemoryServer.create({
        instance: {
            ip: domain,
            port: Number(port),
        },
    });

    await mongoose.connect(`${config.memoryServer.url}/${config.memoryServer.name}`);
    await mongoose.connection.db.dropDatabase();

    await mongoose.disconnect();
};
