import { KinesisStreamEvent } from 'aws-lambda';
import { interactionUpdatedHandler } from '../../../lib/metrics/interactions/eventHandlers/interactionUpdatedHandler';
import { interactionCreatedHandler } from '../../../lib/metrics/interactions/eventHandlers/interactionCreatedHandler';
import { MongoInteractionMetricModel } from '@agyletime/performance-utils';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { V2MongoInteractionMetricModel } from '@agyletime/performance-utils';
import { v2InteractionCreatedHandler } from '../../../lib/metrics/interactions/eventHandlers/v2InteractionCreatedHandler';
import { v2InteractionUpdatedHandler } from '../../../lib/metrics/interactions/eventHandlers/v2InteractionUpdatedHandler';

const organisationId = 472398424333;

describe('the series of events', () => {
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        data: {
                            interactionBefore: {
                                _id: '65b85d2b5b7c947d8df7197a',
                                organisationId,
                                identifier: 'ticketEvent:28191618339737',
                                __v: 0,
                                agentAlias: '1901111382168',
                                asOrganisationId: '9',
                                asTeamId: '6',
                                asUserId: '178867',
                                asWorkloadTaskId: '332',
                                asWorkstreamDefinitionId: '97',
                                createdAt: '2024-01-30T02:21:31.768Z',
                                description: 'Ticket updated',
                                endTime: '2024-01-30T02:20:29Z',
                                integrationDetails: {
                                    integrationId: 4,
                                    integrationName: 'Zendesk',
                                    siteId: '6026fc21eaa428de25acc370',
                                    qmIntegrationId: '655acd626bec2795151d2e3d',
                                },
                                isAssign: false,
                                isPrivateComment: false,
                                isPublicComment: true,
                                isReassign: false,
                                isStatusChange: true,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:18:26Z',
                                statusChangeFrom: 'open',
                                statusChangeTo: 'solved',
                                teamId: 6,
                                type: 'worklog',
                                updatedAt: '2024-01-30T02:21:31.768Z',
                                userId: 178867,
                                workEffortSecs: 123,
                                workloadTaskId: 332,
                                workstreamDefinitionId: 97,
                                workstreamIdentifier: '10005172',
                                integration: {
                                    integrationId: 4,
                                    integrationName: 'Zendesk',
                                    siteId: '6026fc21eaa428de25acc370',
                                    qmIntegrationId: '655acd626bec2795151d2e3d',
                                },
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                            },
                            interactionAfter: {
                                organisationId,
                                identifier: 'ticketEvent:28191618339737',
                                __v: 0,
                                agentAlias: '1901111382168',
                                createdAt: '2024-01-30T02:21:31.768Z',
                                description: 'Ticket updated',
                                endTime: '2024-01-30T02:20:29Z',
                                isAssign: false,
                                isPrivateComment: false,
                                isPublicComment: true,
                                isReassign: false,
                                isStatusChange: true,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:20:26Z',
                                statusChangeFrom: 'open',
                                statusChangeTo: 'solved',
                                type: 'worklog',
                                updatedAt: '2024-01-30T02:22:28.46Z',
                                userId: 178867,
                                workEffortSecs: 3,
                                workloadTaskId: 128,
                                workstreamDefinitionId: 152,
                                workstreamIdentifier: '10005172',
                                integration: {
                                    integrationId: 4,
                                    integrationName: 'Zendesk',
                                    siteId: '6026fc21eaa428de25acc370',
                                    qmIntegrationId: '655acd626bec2795151d2e3d',
                                },
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                                workstreamDefinition: {
                                    id: 152,
                                    name: 'Phone Channel Workstream',
                                },
                            },
                            updates: [
                                {
                                    op: 'replace',
                                    path: '/startTime',
                                    value: '2024-01-30T02:20:26Z',
                                },
                                {
                                    op: 'replace',
                                    path: '/updatedAt',
                                    value: '2024-01-30T02:22:28.46Z',
                                },
                                {
                                    op: 'replace',
                                    path: '/workEffortSecs',
                                    value: 3,
                                },
                                {
                                    op: 'replace',
                                    path: '/workstreamDefinitionId',
                                    value: 152,
                                },
                                {
                                    op: 'replace',
                                    path: '/workloadTaskId',
                                    value: 128,
                                },
                            ],
                            operation: 'update',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    const eventTwo: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        data: {
                            interactionBefore: {
                                _id: '65b85d3c5b7c947d8df71ec8',
                                identifier: '28191518178841',
                                organisationId,
                                __v: 0,
                                agentAlias: '1901111382168',
                                asOrganisationId: '9',
                                asTeamId: '6',
                                asUserId: '178867',
                                asWorkloadTaskId: '332',
                                asWorkstreamDefinitionId: '97',
                                completionStatus: 'completed',
                                consultationTimeSecs: 0,
                                contactIdentifier: '28191508426905',
                                createdAt: '2024-01-30T02:21:48.776Z',
                                description: 'Inbound Phone (completed)',
                                endTime: '2024-01-30T02:20:26Z',
                                holdTimeSecs: 0,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:18:17Z',
                                talkTimeSecs: 70,
                                teamId: 6,
                                type: 'Inbound_Phone',
                                updatedAt: '2024-01-30T02:21:48.776Z',
                                userId: 178867,
                                workEffortSecs: 124,
                                workloadTaskId: 332,
                                workstreamDefinitionId: 97,
                                workstreamIdentifier: '10005172',
                                wrapTimeSecs: 54,
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                            },
                            interactionAfter: {
                                identifier: '28191518178841',
                                organisationId,
                                __v: 0,
                                agentAlias: '1901111382168',
                                completionStatus: 'completed',
                                consultationTimeSecs: 0,
                                contactIdentifier: '28191508426905',
                                createdAt: '2024-01-30T02:21:48.776Z',
                                description: 'Inbound Phone (completed)',
                                endTime: '2024-01-30T02:20:26Z',
                                holdTimeSecs: 0,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:18:17Z',
                                talkTimeSecs: 70,
                                type: 'Inbound_Phone',
                                updatedAt: '2024-01-30T02:22:54.35Z',
                                userId: 178867,
                                workEffortSecs: 124,
                                workloadTaskId: 128,
                                workstreamDefinitionId: 152,
                                workstreamIdentifier: '10005172',
                                wrapTimeSecs: 54,
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                                workstreamDefinition: {
                                    id: 152,
                                    name: 'Phone Channel Workstream',
                                },
                            },
                            updates: [
                                {
                                    op: 'replace',
                                    path: '/updatedAt',
                                    value: '2024-01-30T02:22:54.35Z',
                                },
                                {
                                    op: 'replace',
                                    path: '/workstreamDefinitionId',
                                    value: 152,
                                },
                                {
                                    op: 'replace',
                                    path: '/workloadTaskId',
                                    value: 128,
                                },
                            ],
                            operation: 'update',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    const eventThree: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        data: {
                            interaction: {
                                organisationId,
                                identifier: 'ticketEvent:28191618339737',
                                __v: 0,
                                agentAlias: '1901111382168',
                                createdAt: '2024-01-30T02:21:31.768Z',
                                description: 'Ticket updated',
                                endTime: '2024-01-30T02:20:29Z',
                                isAssign: false,
                                isPrivateComment: false,
                                isPublicComment: true,
                                isReassign: false,
                                isStatusChange: true,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:18:26Z',
                                statusChangeFrom: 'open',
                                statusChangeTo: 'solved',
                                type: 'worklog',
                                updatedAt: '2024-01-30T02:21:31.768Z',
                                userId: 178867,
                                workEffortSecs: 123,
                                workloadTaskId: 332,
                                workstreamDefinitionId: 97,
                                workstreamIdentifier: '10005172',
                                integration: {
                                    integrationId: 4,
                                    integrationName: 'Zendesk',
                                    siteId: '6026fc21eaa428de25acc370',
                                    qmIntegrationId: '655acd626bec2795151d2e3d',
                                },
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                                workstreamDefinition: {
                                    id: 97,
                                    name: 'Catch all',
                                },
                            },
                            operation: 'insert',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    const eventFour: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        data: {
                            interaction: {
                                identifier: '28191518178841',
                                organisationId,
                                __v: 0,
                                agentAlias: '1901111382168',
                                completionStatus: 'completed',
                                consultationTimeSecs: 0,
                                contactIdentifier: '28191508426905',
                                createdAt: '2024-01-30T02:21:48.776Z',
                                description: 'Inbound Phone (completed)',
                                endTime: '2024-01-30T02:20:26Z',
                                holdTimeSecs: 0,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2024-01-30T02:18:17Z',
                                talkTimeSecs: 70,
                                type: 'Inbound_Phone',
                                updatedAt: '2024-01-30T02:21:48.776Z',
                                userId: 178867,
                                workEffortSecs: 124,
                                workloadTaskId: 332,
                                workstreamDefinitionId: 97,
                                workstreamIdentifier: '10005172',
                                wrapTimeSecs: 54,
                                agent: {
                                    integrationAlias: '1901111382168',
                                    wfmId: 178867,
                                    emailAddress: 'kpeacey+zd@playvox.com',
                                    timezone: 'Australia/Sydney',
                                    teamId: 6,
                                },
                                workstreamDefinition: {
                                    id: 97,
                                    name: 'Catch all',
                                },
                            },
                            operation: 'insert',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    it('produces the expected results when in order', async () => {
        // Clear the DB before we run this test
        await MongoInteractionMetricModel.deleteMany({ organisationId });

        await interactionCreatedHandler(eventThree);
        await interactionCreatedHandler(eventFour);

        await interactionUpdatedHandler(eventOne);
        await interactionUpdatedHandler(eventTwo);

        const dataInMongo = await MongoInteractionMetricModel.find({ organisationId }).lean();

        // Expect there to be 2 workstreams
        expect(dataInMongo.length).toEqual(2);

        // Expect the definitions to be valid
        expect(dataInMongo[0]!.workstreamDefinitionId).toEqual(152);
        expect(dataInMongo[0]!.workstreamDefinitionName).toEqual('Phone Channel Workstream');

        expect(dataInMongo[1]!.workstreamDefinitionId).toEqual(152);
        expect(dataInMongo[1]!.workstreamDefinitionName).toEqual('Phone Channel Workstream');

        // Expect the metrics to be correct
        expect(dataInMongo[0]!.numberOfInteractions_all).toEqual(1);
        expect(dataInMongo[1]!.numberOfInteractions_all).toEqual(1);
    });

    // Test currently failing because the change has been rolled back
    it.skip('produces the expected results when out of order', async () => {
        // Clear the DB before we run this test
        await MongoInteractionMetricModel.deleteMany({ organisationId });

        await interactionUpdatedHandler(eventOne);
        await interactionUpdatedHandler(eventTwo);

        await interactionCreatedHandler(eventThree);
        await interactionCreatedHandler(eventFour);

        const dataInMongo = await MongoInteractionMetricModel.find({ organisationId }).lean();

        // Expect there to be 2 workstreams
        expect(dataInMongo.length).toEqual(2);

        // Expect the definitions to be valid
        expect(dataInMongo[0]!.workstreamDefinitionId).toEqual(152);
        expect(dataInMongo[0]!.workstreamDefinitionName).toEqual('Phone Channel Workstream');

        expect(dataInMongo[1]!.workstreamDefinitionId).toEqual(152);
        expect(dataInMongo[1]!.workstreamDefinitionName).toEqual('Phone Channel Workstream');

        // Expect the metrics to be correct
        expect(dataInMongo[0]!.numberOfInteractions_all).toEqual(1);
        expect(dataInMongo[1]!.numberOfInteractions_all).toEqual(1);
    });

    it('writes to both collections', async () => {
        await V2MongoInteractionMetricModel.deleteMany({ organisationId });

        await v2InteractionCreatedHandler(eventThree);

        const dataAfterCreate = await V2MongoInteractionMetricModel.find({ organisationId }).lean();

        expect(dataAfterCreate[0]!.interactions[0]!.workstreamDefinitionId).toEqual(97);

        await v2InteractionUpdatedHandler(eventOne);

        const dataAfterUpdate = await V2MongoInteractionMetricModel.find({ organisationId }).lean();

        expect(dataAfterUpdate[0]!.interactions[0]!.workstreamDefinitionId).toEqual(152);
    });
});
