import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../utils/encodeKinesisBody';
import { MongoInteractionsWorkloadMetric, MongoInteractionsWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../lib/userCache/UserCacheModel';
import { interactionCreatedHandler } from '../../../lib/metrics/eventHandlers/interactionCreatedHandler';
import { MongoInteractionMetric, MongoInteractionMetricModel } from '@agyletime/performance-utils';
import { EventDebugLog, EventDebugLogModel } from '../../../lib/models';

const organisationId = 9898989898;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};

describe('Shift Created Handler', () => {
    beforeAll(() => {
        process.env['DEBUG_EVENT_COLLECTION'] = 'true';
    });
    afterAll(() => {
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete process.env['DEBUG_EVENT_COLLECTION'];
    });
    const workEffortSecs_eventOne = 32323;
    const workloadTaskId_eventOne = 123;
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        agyletimeorganisationId: organisationId.toString(),
                        data: {
                            interaction: {
                                organisationId: organisationId,
                                identifier: 'ticketAudit:wat24592740335129_1698592078234',
                                __v: 0,
                                agentAlias: 'wat20274906959769',
                                contactIdentifier: 'wat2310.11354293.Tu5WwJsl4WWJ7',
                                createdAt: '2023-10-29T20:11:54.673Z',
                                description: 'Chat Message',
                                endTime: '2023-10-29T15:07:58.234Z',
                                isPrivateComment: false,
                                isPublicComment: false,
                                isReassign: false,
                                isStatusChange: false,
                                pvSchemaComment: 'wa-cmd2-interaction',
                                startTime: '2023-10-29T15:07:58.234Z',
                                type: 'chat',
                                updatedAt: '2023-10-29T20:11:54.673Z',
                                userId: 1499,
                                workEffortSecs: workEffortSecs_eventOne,
                                workloadTaskId: workloadTaskId_eventOne,
                                workstreamDefinitionId: 5,
                                workstreamIdentifier: 'wat2154',
                                agent: {
                                    integrationAlias: 'wat20274906959769',
                                    wfmId: 1499,
                                    emailAddress: 'jpromero@playvox.com',
                                    timezone: 'America/New_York',
                                    teamId: 3,
                                },
                                workstreamDefinition: {
                                    id: 5,
                                    name: 'Catch All 1',
                                },
                            },
                            operation: 'insert',
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Single event', () => {
        let workloadDataInMongo: MongoInteractionsWorkloadMetric[] = [];
        let interactionsDataInMongo: MongoInteractionMetric[] = [];
        let eventDebugLog: EventDebugLog[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            // Clear the DB before we run this test
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
            await MongoInteractionMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });

            await interactionCreatedHandler(eventOne);
            workloadDataInMongo = await MongoInteractionsWorkloadMetricModel.find({ organisationId }).lean();
            interactionsDataInMongo = await MongoInteractionMetricModel.find({ organisationId }).lean();
            eventDebugLog = await EventDebugLogModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await MongoInteractionsWorkloadMetricModel.deleteMany({ organisationId });
            await MongoInteractionMetricModel.deleteMany({ organisationId });
            await EventDebugLogModel.deleteMany({ organisationId });
        });

        it('calculates total handle time correctly', () => {
            // Expect there to be 2 workstreams
            expect(workloadDataInMongo).toBeDefined();
            const expectedMetric = workloadDataInMongo.find((d) => d.workloadTaskId === workloadTaskId_eventOne);
            expect(expectedMetric?.totalHandleTime_interactions).toBe(workEffortSecs_eventOne);
            expect(interactionsDataInMongo).toBeDefined();
            expect(interactionsDataInMongo.length).toBeGreaterThan(0);
            expect(eventDebugLog.length).toBeGreaterThan(0);
        });
    });
});
