import { KinesisStreamEvent } from 'aws-lambda';
import encodeKinesisBody from '../../../utils/encodeKinesisBody';
import { taskDataMergedHandler } from '../../../../lib/metrics/workload/eventHandlers/taskDataMergedHandler';
import { MongoActualHoursWorkloadMetric, MongoActualHoursWorkloadMetricModel } from '@agyletime/performance-utils';
import { UserCacheModel, UserCacheType } from '../../../../lib/userCache/UserCacheModel';
import {
    TaskDefinitionModel,
    TaskDefinitionMongoSchemaWithLastOccurance,
} from '@agyletime/performance-utils/lib/mongo/models';
import { TaskDefinitionMongoSchema } from '@agyletime/wfm-types/lib/schedule/taskDefinition';
import { differenceInSeconds } from 'date-fns';

const organisationId = 512324124124;
const userOne: Partial<UserCacheType> = {
    id: 73185,
    timezone: 'America/New_York',
    team: {
        id: 1,
    },
    email: 'a@b.com',
    organisationId,
};
const taskDefinition_isAvailableOccupancy: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is Available Occupancy Task Task',
    identifier: 'isAvailableOccupancy',
    id: 12345,
    description: 'Is Available Occupancy Task Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: true,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: false,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};
const taskDefinition_isAfterCallWork: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is After Call Work Task',
    identifier: 'isAfterCallWork',
    id: 123456,
    description: 'Is After Call Work Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: true,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};
const taskDefinition_forNoEndtime: Partial<TaskDefinitionMongoSchema> = {
    name: 'Is After Call Work Task',
    identifier: 'isAfterCallWork',
    id: 1234567,
    description: 'Is After Call Work Task',
    organisationId,
    isPaid: false,
    isAvailable: false,
    isAvailableOccupancy: false,
    isPlanned: false,
    isBreak: false,
    isDefaultWorkloadTask: false,
    includeInTimeOffLimits: false,
    isIdentifierRegEx: false,
    isProductive: false,
    isLeave: false,
    isLogout: true,
    hideLeaveInAgentViews: false,
    isShiftSwapAllowed: 'No',
    isAfterCallWork: true,
    isMergeAllowed: true,
    colour: '00000',
    disableHandleTimeClock: false,
    additionalHoursOverride: false,
    defaultDuration: 30,
    businessRoles: [],
};

describe('the series of events', () => {
    const isAvailableOccupancyTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T00:58:42.000Z',
        endTime: '2023-12-01T02:58:42.000Z',
        taskDefinitionId: taskDefinition_isAvailableOccupancy.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const isAfterCallWorkTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T02:58:42.000Z',
        endTime: '2023-12-01T05:58:42.000Z',
        taskDefinitionId: taskDefinition_isAfterCallWork.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const noEndTimeTask = {
        _id: '65692fd72d1f23f4830597c3',
        organisationId: organisationId,
        identifier: '44f637b4-6971-9807-5637-938fdbb58406',
        __v: 0,
        agentAlias: '0053t000009YK6MAAW',
        createdAt: '2023-12-01T00:59:03.430Z',
        shiftId: '6566b308d2e21a9b0c9f0be6',
        startTime: '2023-12-01T02:58:42.000Z',
        taskDefinitionId: taskDefinition_forNoEndtime.id,
        taskTypeIdentifier: 'OFFLINE',
        updatedAt: '2023-12-01T00:59:03.430Z',
        userEmail: 'wfm.test@playvox.com',
        userId: userOne.id,
    };
    const eventOne: KinesisStreamEvent = {
        Records: [
            {
                kinesis: {
                    data: encodeKinesisBody({
                        plvxwfmid: organisationId.toString(),
                        data: {
                            shiftId: '6566b308d2e21a9b0c9f0be6',
                            userId: userOne.id,
                            tasks: [isAvailableOccupancyTask, isAfterCallWorkTask, noEndTimeTask],
                        },
                    }),
                },
            },
        ],
    } as KinesisStreamEvent;

    describe('Task Data Handler', () => {
        let dataInMongo: MongoActualHoursWorkloadMetric[] = [];
        let taskDefinitionsInMongo: TaskDefinitionMongoSchemaWithLastOccurance[] = [];
        beforeAll(async () => {
            await UserCacheModel.insertMany([userOne]);
            await TaskDefinitionModel.insertMany([
                taskDefinition_isAvailableOccupancy,
                taskDefinition_isAfterCallWork,
                taskDefinition_forNoEndtime,
            ]);
            // Clear the DB before we run this test
            await MongoActualHoursWorkloadMetricModel.deleteMany({ organisationId });

            await taskDataMergedHandler(eventOne);
            dataInMongo = await MongoActualHoursWorkloadMetricModel.find({ organisationId }).lean();
            taskDefinitionsInMongo = await TaskDefinitionModel.find({ organisationId }).lean();
        });
        afterAll(async () => {
            await UserCacheModel.deleteMany({ organisationId });
            await TaskDefinitionModel.deleteMany({ organisationId });
            await MongoActualHoursWorkloadMetricModel.deleteMany({ organisationId });
        });

        for (const taskDefinition of [
            taskDefinition_isAvailableOccupancy,
            taskDefinition_isAfterCallWork,
            taskDefinition_forNoEndtime,
        ]) {
            const todaysDate = new Date().toISOString().split('T')[0];
            it(`updates the lastOccurance on the task definition "${taskDefinition.name}" in cache to ${todaysDate}`, () => {
                const actualTaskDefinition = taskDefinitionsInMongo.find((t) => t.id === taskDefinition.id);
                expect(actualTaskDefinition).toBeDefined();
                expect(actualTaskDefinition?.lastOccurance).toBeDefined();
                const taskDefinitionDate = actualTaskDefinition?.lastOccurance.toISOString().split('T')[0];
                expect(taskDefinitionDate).toBe(todaysDate);
            });
        }

        it('calculates actual available time correctly', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            const exepctedActualAvailableTime = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.actualAvailableTime_actualHours).toBe(exepctedActualAvailableTime);
            const isAfterCallWorkTask = dataInMongo.find((t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id);
            expect(isAfterCallWorkTask?.actualAvailableTime_actualHours).toBe(0);
        });
        it('calculates actual after call work time correctly', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            expect(isAvailableOccupancyMetric?.actualAfterCallWorkTime_actualHours).toBe(0);
            const exepctedAfterCallWorkTime = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            const isAfterCallWorkMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            expect(isAfterCallWorkMetric?.actualAfterCallWorkTime_actualHours).toBe(exepctedAfterCallWorkTime);
        });
        it('calculates total actual hours', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const isAvailableOccupancyMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAvailableOccupancy.id,
            );
            let exepctedTotalActualHours = differenceInSeconds(
                new Date(isAvailableOccupancyTask.endTime),
                new Date(isAvailableOccupancyTask.startTime),
            );
            expect(isAvailableOccupancyMetric?.totalActualTasks_actualHours).toBe(exepctedTotalActualHours);

            const isAfterCallWorkMetric = dataInMongo.find(
                (t) => t.workloadTaskId === taskDefinition_isAfterCallWork.id,
            );
            exepctedTotalActualHours = differenceInSeconds(
                new Date(isAfterCallWorkTask.endTime),
                new Date(isAfterCallWorkTask.startTime),
            );
            expect(isAfterCallWorkMetric?.totalActualTasks_actualHours).toBe(exepctedTotalActualHours);
        });
        it('should not have generated metric for task with no endtime', () => {
            expect(dataInMongo).toBeDefined();
            expect(dataInMongo.length).toBeGreaterThan(0);
            const noEndTimeMetric = dataInMongo.find((t) => t.workloadTaskId === taskDefinition_forNoEndtime.id);
            expect(noEndTimeMetric).toBeUndefined();
        });
    });
});
