import calculateActualCreateDiff from '../../../lib/metrics/actuals/calculateActualCreateDiff';
import type { ActualFilterParams, TaskDataMergedEvent } from '../../../lib/metrics';
import { TaskDefinitionMongoSchema } from '@agyletime/wfm-types/lib/schedule/taskDefinition';
import dateFromParts from '../../utils/dateFromParts';
import { toZonedTime } from 'date-fns-tz';

const taskDefinition: TaskDefinitionMongoSchema = {
    id: 1,
    identifier: 'task identifier',
    name: 'task name',
    organisationId: 0,
};

const taskDefinitionsMap = new Map([[taskDefinition.id, taskDefinition]]);

const params: ActualFilterParams = {
    organisationId: 0,
    shiftId: '123',
    taskDefinitionsMap,
    teamId: 2,
    timezone: 'America/New_York',
    userId: 4,
};
const shift: TaskDataMergedEvent = {
    shiftId: '123',
    tasks: [
        {
            organisationId: 0,
            identifier: 'shift id',
            agentAlias: 'alias',
            createdAt: '2023-03-03',
            shiftId: '123',
            startTime: dateFromParts({ year: 2024, month: 3, day: 17 }).toISOString(),
            endTime: dateFromParts({ year: 2024, month: 3, day: 21 }).toISOString(),
            taskDefinitionId: taskDefinition.id,
            taskTypeIdentifier: 'string',
            updatedAt: '2024-01-01',
            userEmail: 'string',
            userId: 4,
        },
        {
            organisationId: 0,
            identifier: 'shift id',
            agentAlias: 'alias',
            createdAt: '2023-03-04',
            shiftId: '123',
            startTime: dateFromParts({ year: 2024, month: 3, day: 12 }).toISOString(),
            endTime: dateFromParts({ year: 2024, month: 3, day: 20 }).toISOString(),
            taskDefinitionId: taskDefinition.id,
            taskTypeIdentifier: 'string',
            updatedAt: '2024-01-01',
            userEmail: 'string',
            userId: 4,
        },
        {
            organisationId: 0,
            identifier: 'shift id',
            agentAlias: 'alias',
            createdAt: '2023-03-04',
            shiftId: '123',
            startTime: dateFromParts({ year: 2024, month: 3, day: 18 }).toISOString(),
            endTime: dateFromParts({ year: 2024, month: 3, day: 22 }).toISOString(),
            taskDefinitionId: taskDefinition.id,
            taskTypeIdentifier: 'string',
            updatedAt: '2024-01-01',
            userEmail: 'string',
            userId: 4,
        },
    ],
    userId: 4,
};

describe('calculateActualCreateDiff', () => {
    it('saves the correct earliestShiftTaskAsLocal', () => {
        const result = calculateActualCreateDiff({ shift, params });

        const expectedEarliestShift = toZonedTime(shift.tasks[1]!.startTime, params.timezone);
        expect(result.replacement.earliestShiftTaskAsLocal).toEqual(expectedEarliestShift);
    });

    it('saves the correct latestShiftTaskAsLocal', () => {
        const result = calculateActualCreateDiff({ shift, params });

        const expectedLatestShift = toZonedTime(shift.tasks[2]!.endTime!, params.timezone);
        expect(result.replacement.latestShiftTaskAsLocal).toEqual(expectedLatestShift);
    });
});
