import type { SQSEvent } from 'aws-lambda';
import { singleSyncPerformanceViews } from '../../../../../lib/metrics/agent/workstream/workers/singleSyncPerformanceViews';
import * as MongoDB from '@agyletime/common-utils/mongo/mongodb'; // important: Make sure not to use the index file or jest.spyOn will break :)

// eslint-disable-next-line @typescript-eslint/no-var-requires,@typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
const dbPool = require('@agyletime/wfm-rds-utils').multiConnectionPool;
// eslint-disable-next-line @typescript-eslint/no-var-requires,@typescript-eslint/no-unsafe-assignment
const doSyncPerformanceViewsModule = require('../../../../../lib/metrics/agent/workstream/doSyncPerformanceViews');

const event = {
    Records: [
        {
            messageId: '1',
            attributes: {
                ApproximateReceiveCount: 1,
            },
            body: JSON.stringify({
                agyletimeorganisationId: 42,
                data: {
                    fromDate: 'from1',
                    toDate: 'to1',
                },
            }),
        },
        {
            messageId: '2',
            attributes: {
                ApproximateReceiveCount: 1,
            },
            body: JSON.stringify({
                agyletimeorganisationId: 43,
                data: {
                    fromDate: 'from2',
                    toDate: 'to2',
                },
            }),
        },
    ],
} as unknown as SQSEvent;

describe('Handle inbound SQS records worker - singleSyncPerformanceViews', () => {
    afterEach(() => {
        jest.resetAllMocks();
    });

    it('should invoke processInboundSQSEventRecords and call doSyncPerformanceViews', async () => {
        const init = jest.spyOn(dbPool, 'init').mockImplementation(() => Promise.resolve());
        const doSyncPerformanceViewsMock = jest
            .spyOn(doSyncPerformanceViewsModule, 'doSyncPerformanceViews')
            .mockImplementation(() => Promise.resolve());
        const mysqlCloseAll = jest.spyOn(dbPool, 'closeAll').mockImplementation(() => Promise.resolve());
        const mongoCloseConn = jest.spyOn(MongoDB, 'closeConnection').mockImplementation(() => Promise.resolve());

        await singleSyncPerformanceViews(event);

        expect(init).toHaveBeenCalledWith();
        expect(doSyncPerformanceViewsMock).toHaveBeenCalledTimes(2);
        // expect(invoke).toBeCalledWith({
        //     fnToExecute: doSyncPerformanceViews,
        //     event,
        //     queueUrl: config.performance.syncPerformanceViewsQueueUrl
        // })
        expect(mysqlCloseAll).toHaveBeenCalled();
        expect(mongoCloseConn).toHaveBeenCalled();
    });

    it('should catch an error and close the database connections', async () => {
        const init = jest.spyOn(dbPool, 'init').mockImplementation(() => Promise.resolve());
        const doSyncPerformanceViewsMock = jest
            .spyOn(doSyncPerformanceViewsModule, 'doSyncPerformanceViews')
            .mockImplementation(() => Promise.reject(new Error('test')));
        const mysqlCloseAll = jest.spyOn(dbPool, 'closeAll').mockImplementation(() => Promise.resolve());
        const mongoCloseConn = jest.spyOn(MongoDB, 'closeConnection').mockImplementation(() => Promise.resolve());

        await singleSyncPerformanceViews(event);

        expect(init).toHaveBeenCalledWith();
        expect(doSyncPerformanceViewsMock).toHaveBeenCalledTimes(2);
        expect(mysqlCloseAll).toHaveBeenCalled();
        expect(mongoCloseConn).toHaveBeenCalled();
    });

    it('should throw an error and not process any messages if DB opening fails', async () => {
        const init = jest.spyOn(dbPool, 'init').mockImplementation(() => Promise.reject(new Error('test')));
        const doSyncPerformanceViewsMock = jest
            .spyOn(doSyncPerformanceViewsModule, 'doSyncPerformanceViews')
            .mockImplementation(() => Promise.resolve());
        const mysqlCloseAll = jest.spyOn(dbPool, 'closeAll').mockImplementation(() => Promise.resolve());
        const mongoCloseConn = jest.spyOn(MongoDB, 'closeConnection').mockImplementation(() => Promise.resolve());

        try {
            await singleSyncPerformanceViews(event);
        } catch (e: unknown) {
            /* intentionally empty */
        }

        expect(init).toHaveBeenCalledWith();
        expect(doSyncPerformanceViewsMock).toHaveBeenCalledTimes(0);
        expect(mysqlCloseAll).toHaveBeenCalled();
        expect(mongoCloseConn).toHaveBeenCalled();
    });
});
