import { log } from '../../../lib/utils';
import { extractInteractionEventParams } from '../../../lib/metrics/interactions/extractInteractionEventParams';

const logWarnSpy = jest.spyOn(log, 'warn');
describe('extractInteractionEventParams', () => {
    it('should expose a function', () => {
        expect(extractInteractionEventParams).toBeDefined();
    });

    it('extractInteractionEventParams should return expected output', async () => {
        const interaction = {
            type: 'test type',
            startTime: new Date().toISOString(),
            workstreamIdentifier: 'identifier',
            organisationId: 1,
            agent: {
                wfmId: 1,
                teamId: 1,
                timezone: 'Australia/Sydney',
                integrationAlias: 'alias',
            },
            identifier: 'identifier',
            createdAt: new Date().toISOString(),
            workloadTaskId: 1,
        };
        const retValue = await extractInteractionEventParams(interaction);
        expect(retValue).toBeTruthy();
        expect(logWarnSpy).not.toHaveBeenCalled();
    });
    it('extractInteractionEventParams logs warning and doesn not return params', async () => {
        const interaction = {
            type: 'test type',
            startTime: new Date().toISOString(),
            workstreamIdentifier: 'identifier',
            organisationId: 1,
            agent: {
                teamId: 1,
                timezone: 'Australia/Sydney',
                integrationAlias: 'alias',
            },
            identifier: 'identifier',
            createdAt: new Date().toISOString(),
            workloadTaskId: 1,
        };
        const retValue = await extractInteractionEventParams(interaction);
        expect(retValue).toBeFalsy();
        expect(logWarnSpy).toHaveBeenCalled();
    });
});
