import * as mongodb from '../../../../lib/utils/mongodb';
import { makeToken, tokenData } from '../../../utils/makeToken';
import { APIGatewayProxyEvent } from 'aws-lambda';
import { AthenaQueryMetadataModel, AthenaQueryResponse } from '../../../../lib/athena';
import { reconcileInteractionsHandler } from '../../../../lib/metrics/interactions/reconciliationHandlers/reconcileInteractionsHandler';
import { extractIdentity } from '@agyletime/wfm-aws-utils';
import * as initiateAthenaQuery from '../../../../lib/athena/utils';

jest.mock('../../../../lib/utils/mongodb');

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        extractIdentity: jest.fn(),
    };
});

const organisationId = '9';
const token = makeToken(tokenData);
let apiGatewayEvent: APIGatewayProxyEvent;
let mockWriteAthenaQueryMetadata: unknown;

describe('Reconcile interaction handler', () => {
    beforeEach(() => {
        mockWriteAthenaQueryMetadata = jest.spyOn(AthenaQueryMetadataModel, 'create').mockImplementation();
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
        jest.spyOn(initiateAthenaQuery, 'initiateAthenaQuery').mockImplementation(
            (): Promise<AthenaQueryResponse> => Promise.resolve({ queryExecutionId: '123' }),
        );
        (extractIdentity as jest.Mock).mockImplementation(() => ({
            organisationId: 9,
            identifier: 'admin@agyletime.com',
        }));
        apiGatewayEvent = {
            headers: {
                Authorization: `Bearer ${token}`,
            },
            queryStringParameters: {},
            identity: {
                accessKey: null,
                accountId: 'offlineContext_accountId',
                apiKey: 'offlineContext_apiKey',
                caller: 'offlineContext_caller',
                cognitoAuthenticationProvider: 'offlineContext_cognitoAuthenticationProvider',
                cognitoAuthenticationType: 'offlineContext_cognitoAuthenticationType',
                cognitoIdentityId: 'offlineContext_cognitoIdentityId',
                cognitoIdentityPoolId: 'offlineContext_cognitoIdentityPoolId',
                organisationId: organisationId.toString(),
                principalOrgId: null,
                sourceIp: '127.0.0.1',
                user: 'offlineContext_user',
                userAgent:
                    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36',
                userArn: 'offlineContext_userArn',
            },
            requestContext: {
                authorizer: { organisationId: organisationId, identifier: 'admin@agyletime.com', userId: 1 },
                httpMethod: 'POST',
                // @ts-expect-error Identity isn't implemented here since it's not used
                identity: {},
                path: '/performance/metrics/agent/workstream',
                protocol: 'HTTP/1.1',
                resourcePath: '/metrics/agent/workstream',
                stage: 'dev',
            },
            timestamp: '2020-08-26T22:49:48.521Z',
        };
    });

    it('returns 200 for a successful query', async () => {
        const mockPostBody = {};
        apiGatewayEvent.body = JSON.stringify(mockPostBody);

        const response = await reconcileInteractionsHandler(apiGatewayEvent);

        // Expect an OK response
        expect(response.statusCode).toEqual(200);

        // Expect the athena query to be logged
        expect(mockWriteAthenaQueryMetadata).toHaveBeenCalled();
    });
});
