import { reconcileRosteredMetricsHandler } from '../../../../lib/metrics/rostered/reconciliationHandlers/reconcileRosteredMetricsHandler';
import * as wfmUtils from '@agyletime/wfm-aws-utils';
import { ReconciliationScheduleJobEvent } from '../../../../lib/metrics';

jest.mock('@agyletime/wfm-aws-utils');
let invokeStepFunctionsSpy: jest.SpyInstance;
describe('Reconcile rostered metrics handler', () => {
    beforeEach(() => {
        jest.spyOn(wfmUtils, 'getSSMParameters').mockImplementation(
            (): Promise<wfmUtils.SSMParameters> => Promise.resolve([{ id: 9, name: 'dummy org' }]),
        );
        jest.spyOn(wfmUtils, 'extractIdentity').mockImplementation(
            (): wfmUtils.AgyleTimeIdentity => ({ organisationId: 9 }),
        );
        // @ts-expect-error incorrect return type
        invokeStepFunctionsSpy = jest.spyOn(wfmUtils, 'invokeStepFunction').mockImplementation(() => Promise.resolve());
    });
    afterEach(() => {
        jest.restoreAllMocks();
    });
    it('returns 200 for a successful query', async () => {
        const event: ReconciliationScheduleJobEvent = {
            periodInHours: '24',
        };

        await reconcileRosteredMetricsHandler(event);

        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
        const toDate = invokeStepFunctionsSpy.mock.calls[0][0].payload.toDate;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
        const expectedFromDate = new Date(toDate);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
        expectedFromDate.setDate(toDate.getHours() - parseInt(event.periodInHours));
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
        expect(invokeStepFunctionsSpy.mock.calls[0][0].payload.fromDate.toISOString()).toBe(
            expectedFromDate.toISOString(),
        );
    });
});
