import type { KinesisStreamEvent } from 'aws-lambda';
import { shiftDeletedHandler } from '../../../lib/metrics/rostered/eventHandlers/shiftDeletedHandler';
import * as mongodb from '../../../lib/utils/mongodb';
import { MongoRosteredMetricModel } from '@agyletime/performance-utils';

jest.createMockFromModule('@agyletime/performance-utils');
jest.mock('../../../lib/utils/mongodb');

const mockResult = 'Success!';

const event = {
    Records: [
        {
            awsRegion: 'string',
            eventID: 'string',
            eventName: 'string',
            eventSource: 'string',
            eventSourceARN: 'string',
            eventVersion: 'string',
            invokeIdentityArn: 'string',
            kinesis: {
                approximateArrivalTimestamp: 123,
                data: 'string',
                kinesisSchemaVersion: 'string',
                partitionKey: 'string',
                sequenceNumber: 'string',
            },
        },
    ],
} as KinesisStreamEvent;

describe('ShiftDeleted', () => {
    beforeEach(() => {
        // @ts-expect-error type mismatch
        jest.spyOn(MongoRosteredMetricModel, 'updateOne').mockImplementation(() => mockResult);
        // @ts-expect-error type mismatch
        jest.spyOn(mongodb, 'init').mockImplementation(() => Promise.resolve());
    });

    afterEach(() => {
        jest.resetAllMocks();
    });

    describe('with a valid event', () => {
        it('throws an error if the db connection fails', async () => {
            jest.spyOn(mongodb, 'init').mockImplementation(() => {
                throw new Error('Failed to connect to MongoDB');
            });

            let error = null;

            try {
                await shiftDeletedHandler(event);
            } catch (e: unknown) {
                error = e;
            }
            expect(error).toBeTruthy();
        });
    });
});
