import { syncUsersToDb } from '../../lib/userCache/syncUsersToDb';
import { getSSMParameters, SSMParametersForOrganisation, getOrganisationApiParamsById } from '@agyletime/wfm-aws-utils';

// jest mocking on
jest.mock('../../lib/userCache/utils/fetchAllUsers');
jest.mock('../../lib/userCache/utils/insertOrUpdateUsers');
jest.mock('../../lib/userHistory/UserHistoryProvider');

jest.mock('@agyletime/wfm-aws-utils', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const original = jest.requireActual('@agyletime/wfm-aws-utils');
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return {
        ...original,
        getSSMParameters: jest.fn(),
        getOrganisationApiParamsById: jest.fn(),
    };
});
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const STUBS: any = {};
// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-var-requires
const { fetchAllUsers } = require('../../lib/userCache/utils/fetchAllUsers');
// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-var-requires
const { insertOrUpdateUsers } = require('../../lib/userCache/utils/insertOrUpdateUsers');
const UserHistoryProvider = jest.createMockFromModule('../../lib/userHistory/UserHistoryProvider');
describe('syncUsersToDb', () => {
    beforeEach(() => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
        fetchAllUsers.mockResolvedValue([{ id: 1 }, { id: 2 }]);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        STUBS.insertOrUpdateUsers = jest.fn().mockResolvedValue({});
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
        insertOrUpdateUsers.mockImplementation(() => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return,@typescript-eslint/no-unsafe-member-access
            return STUBS.insertOrUpdateUsers;
        });
        // @ts-expect-error type mismatch
        // eslint-disable-next-line @typescript-eslint/require-await
        UserHistoryProvider.saveUserHistory = jest.fn(async (users: unknown, updatedAt: unknown) => {
            // eslint-disable-next-line @typescript-eslint/restrict-template-expressions
            console.log(`Mocking results for users: ${users} updatedAt: ${updatedAt}`);
        });

        (getSSMParameters as jest.Mock).mockImplementation(() => [{ id: 9, api: { foo: 'bar' } }]);
        (getOrganisationApiParamsById as jest.Mock).mockImplementation(() => ({
            org: 'params',
        }));
    });
    afterAll(() => {
        jest.resetAllMocks();
    });
    it('should go through the proper flow', async () => {
        // eslint-disable-next-line @typescript-eslint/no-confusing-void-expression
        const result = await syncUsersToDb({
            id: 1,
            api: { agyletimeApiUrl: 'blahblah' },
        } as SSMParametersForOrganisation);
        expect(result).toBeUndefined();
    });
});
