import { fetchAllUsers } from '../../../lib/userCache/utils/fetchAllUsers';
import { config } from '../../../lib/config';
import { OrganisationId } from '@agyletime/wfm-types';
import { BasicAuthApiClientParams } from '@agyletime/wfm-client/lib/types/agyleTimeClient/agyleTimeClient';

// jest mocking on
jest.mock('@agyletime/wfm-client');
// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-var-requires,@typescript-eslint/no-unsafe-member-access
const UserApi = require('@agyletime/wfm-client').UserApi;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const STUBS: any = {};

describe('fetchAllUsers', () => {
    beforeEach(() => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        STUBS.getAllUsersMock = jest.fn().mockResolvedValue(stubUserApiResponses);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-member-access
        UserApi.mockImplementation(() => {
            return {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
                getAllUsers: STUBS.getAllUsersMock,
            };
        });
    });

    it('should fetch from api', async () => {
        config.agyletime.api_gateway_base_url = 'https://api-au.alpha.cloud.agyletime.net';
        const organisationId: OrganisationId = 9002;

        const users = await fetchAllUsers(organisationId, {} as BasicAuthApiClientParams);
        expect(users.length).toEqual(expectedUsers.length);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        expect(STUBS.getAllUsersMock.mock.calls[0]).toEqual([
            true,
            {},
            {
                fields: {
                    team: true,
                    roles: true,
                    tags: true,
                    firstName: true,
                    lastName: true,
                    email: true,
                    timezone: true,
                    id: true,
                    integrations: true,
                    _id: false,
                },
                pageSize: 10000,
            },
        ]);
    });
});
const expectedUsers = [
    {
        firstName: 'Agyle',
        lastName: 'Adminsss',
        email: 'admin@agyletime.com',
        id: 1,
        roles: [{ name: 'Default Business Role', id: 1 }],
        team: { id: 1, name: 'Administrator' },
        timezone: 'Australia/Sydney',
        tags: [],
    },
    {
        email: 'TA1-USR1@t.com',
        firstName: 'TA1-USR1',
        id: 3,
        lastName: 'Test',
        roles: [{ name: 'Default Business Role', id: 1 }],
        tags: [
            {
                text: 'godlike',
            },
            {
                text: 'beyondgodlike',
            },
        ],
        timezone: 'Australia/Perth',
        team: { id: 3, name: 'Team-A1' },
    },
];
const stubUserApiResponses = {
    data: expectedUsers,
};
