import { filterOrgParams, shouldRunSyncForOrgId } from '../../../lib/userCache/utils/filterOrgParams';
import moment from 'moment';
import type { SSMParameters } from '@agyletime/wfm-aws-utils';

jest.mock('moment', () => {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const actualMoment = jest.requireActual('moment');
    // Mocking moment func:
    // moment() => return specific date, and it won't affect moment(date) with param.
    // eslint-disable-next-line @typescript-eslint/no-unsafe-call,@typescript-eslint/no-unsafe-return, @typescript-eslint/no-explicit-any
    const mockMoment: any = (date: string | undefined) => actualMoment(date ?? '2023-06-18T00:00:00.000Z');

    // Now assign all properties from actual moment to the mock moment, so that they can be used normally
    for (const prop in actualMoment) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment,@typescript-eslint/no-unsafe-member-access
        mockMoment[prop] = actualMoment[prop];
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return mockMoment;
});

describe('filterOrgParams', () => {
    afterAll(() => {
        jest.clearAllMocks();
    });
    it('should accept orgId parameter', () => {
        const organisationParameters: SSMParameters = [
            { id: 101, name: 'https://clientA.blah.blah' },
            { id: 102, name: 'https://clientB.blah.blah' },
            { id: 103, name: 'https://clientC.blah.blah' },
            { id: 104, name: 'https://clientD.blah.blah' },
            { id: 105, name: 'https://clientE.blah.blah' },
        ];
        const orgIdToSync = 101;
        const filteredOrgSsmParams = filterOrgParams(organisationParameters, orgIdToSync);
        expect(filteredOrgSsmParams).toEqual([{ id: 101, name: 'https://clientA.blah.blah' }]);
    });

    it('should filter organisations if no orgId provided', () => {
        const organisationParameters: SSMParameters = [
            { id: 101, name: 'https://clientA.blah.blah' },
            { id: 102, name: 'https://clientB.blah.blah' },
            { id: 103, name: 'https://clientC.blah.blah' },
            { id: 104, name: 'https://clientD.blah.blah' },
            { id: 105, name: 'https://clientE.blah.blah' },
        ];
        const orgIdToSync = 0;
        const filteredOrgSsmParams = filterOrgParams(organisationParameters, orgIdToSync);
        expect(filteredOrgSsmParams).toHaveLength(2);
    });

    it('should log warn if no organisation found matching orgId', () => {
        const organisationParameters: SSMParameters = [
            { id: 101, name: 'https://clientA.blah.blah' },
            { id: 102, name: 'https://clientB.blah.blah' },
            { id: 103, name: 'https://clientC.blah.blah' },
            { id: 104, name: 'https://clientD.blah.blah' },
            { id: 105, name: 'https://clientE.blah.blah' },
        ];
        const orgIdToSync = 21;
        const filteredOrgSsmParams = filterOrgParams(organisationParameters, orgIdToSync);
        expect(filteredOrgSsmParams).toHaveLength(0);
    });

    it('should run even or odd orgIds bassed on day of week', () => {
        let day = moment('2023-06-18T00:00:00.000Z').utcOffset(0).day();
        let isValid = shouldRunSyncForOrgId(101, day);
        expect(isValid).toEqual(false);
        day = moment('2023-06-19T00:00:00.000Z').utcOffset(0).day();
        isValid = shouldRunSyncForOrgId(101, day);
        expect(isValid).toEqual(true);
    });
});
