import * as jwt from 'jsonwebtoken';

export interface Token {
    identifier: string;
    organisationId: number;
    roles?: {
        businessRoles: { roleId: number; name: string }[];
    };
    scopes: string;
    userId: number;
    expiryMins?: number;
    data?: JSON;
}

export const makeToken = ({ expiryMins = 60, organisationId, identifier, userId, scopes, data }: Token): string => {
    const headerData: Token = {
        organisationId,
        identifier,
        userId,
        scopes,
        ...data,
    };

    return jwt.sign(
        {
            exp: Math.floor(Date.now() / 1000) + 60 * expiryMins,
            data: headerData,
        },
        // deepcode ignore HardcodedSecret/test: Used for testing
        'jhgcyu7JFTYDRTDS!!@@khjbsdkkjjbhcds]095|',
    ) as unknown as string;
};

export const tokenData: Token = {
    organisationId: 9,
    identifier: 'admin@agyletime.com',
    userId: 1,
    scopes: 'urn:agyletime:workactivity:workstreams,urn:agyletime:workactivity,urn:agyletime:workactivity:update',
    roles: {
        businessRoles: [
            {
                roleId: 4,
                name: 'Administrator',
            },
            {
                roleId: 5,
                name: 'ANZ Consumer Calls',
            },
            {
                roleId: 6,
                name: 'USA Consumer Calls',
            },
            {
                roleId: 9,
                name: 'ANZ Consumer Tickets ',
            },
            {
                roleId: 10,
                name: 'USA Consumer Tickets ',
            },
            {
                roleId: 15,
                name: 'Merchant Disputes',
            },
            {
                roleId: 17,
                name: 'Merchant Admin',
            },
            {
                roleId: 25,
                name: 'Manual ID',
            },
        ],
    },
};
